/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

'use strict';

var util = require('util');
var _ = require('underscore');
var azureCommon = require('azure-common');
var xml = azureCommon.xml2js;
var js2xml = azureCommon.js2xml;
var Service = azureCommon.Service;
var WebResource = azureCommon.WebResource;

var WebSiteManagementClient = ( /** @lends WebSiteManagementClient */ function() {
  /**
   * @class
   * The Web Sites Management API provides a RESTful set of web services that
   * interact with the Windows Azure Web Sites service to manage your web
   * sites. The API has entities that capture the relationship between an end
   * user and Windows Azure Web Sites service.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
   * more information)
   * Initializes a new instance of the WebSiteManagementClient class.
   * @constructor
   * 
   * @param {SubscriptionCloudCredentials} credentials When you create an Azure
   * subscription, it is uniquely identified by a subscription ID. The
   * subscription ID forms part of the URI for every call that you make to the
   * Service Management API. The Azure Service Management API uses mutual
   * authentication of management certificates over SSL to ensure that a
   * request made to the service is secure. No anonymous requests are allowed.
   * 
   * @param {string} [credentials.subscriptionId]
   * 
   * @param {string} [baseUri] The URI used as the base for all Service
   * Management requests.
   * 
   * @param {Array} filters
   */
  function WebSiteManagementClient(credentials, baseUri, filters) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    WebSiteManagementClient['super_'].call(this, credentials, filters);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.core.windows.net';
    }
    /**
     * Provides an instance of the
     * [ServerFarmOperations](-ServerFarmOperations.html) object.
     * @type {object}
     */
    this.serverFarms = new ServerFarmOperations(this);
    /**
     * Provides an instance of the [WebSiteOperations](-WebSiteOperations.html)
     * object.
     * @type {object}
     */
    this.webSites = new WebSiteOperations(this);
    /**
     * Provides an instance of the
     * [WebSpaceOperations](-WebSpaceOperations.html) object.
     * @type {object}
     */
    this.webSpaces = new WebSpaceOperations(this);
  }
  
  util.inherits(WebSiteManagementClient, Service);
  
  /**
   * The Get Operation Status operation returns the status of the specified
   * operation. After calling a long-running operation, you can call Get
   * Operation Status to determine whether the operation has succeeded,
   * failed, timed out, or is still in progress.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/ee460783.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the webspace for the website
   * where the operation was targeted.
   * 
   * @param {string} siteName The name of the site where the operation was
   * targeted.
   * 
   * @param {string} operationId The operation ID for the operation you wish to
   * track. The operation ID is returned in the ID field in the body of the
   * response for long-running operations.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteManagementClient.prototype.getOperationStatus = function(webSpaceName, siteName, operationId, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (siteName === null || siteName === undefined) {
      return callback(new Error('siteName cannot be null.'));
    }
    if (operationId === null || operationId === undefined) {
      return callback(new Error('operationId cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.baseUri;
    var url2 = '/' + (this.credentials.subscriptionId ? this.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + siteName.trim() + '/operations/' + operationId.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { errors: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var operationElement = js2xml.getElement(responseDoc, responseDoc, 'OPERATION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (operationElement) {
          var createdTimeElement = js2xml.getElement(responseDoc, operationElement, 'CREATEDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (createdTimeElement) {
            var createdTimeInstance = new Date(createdTimeElement);
            result.createdTime = createdTimeInstance;
          }
          
          var errorsSequenceElement = js2xml.getElement(responseDoc, operationElement, 'ERRORS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorsSequenceElement) {
            var isNil = false;
            var nilAttribute = js2xml.getAttribute(responseDoc, errorsSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute) {
              isNil = nilAttribute === 'true';
            }
            if (isNil === false) {
              for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, errorsSequenceElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
                var errorsElement = js2xml.getElements(responseDoc, errorsSequenceElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
                var errorInstance = { parameters: [] };
                result.errors.push(errorInstance);
                
                var codeElement = js2xml.getElement(responseDoc, errorsElement, 'CODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (codeElement) {
                  var isNil2 = false;
                  var nilAttribute2 = js2xml.getAttribute(responseDoc, codeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute2) {
                    isNil2 = nilAttribute2 === 'true';
                  }
                  if (isNil2 === false) {
                    var codeInstance = codeElement;
                    errorInstance.code = codeInstance;
                  }
                }
                
                var messageElement = js2xml.getElement(responseDoc, errorsElement, 'MESSAGE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (messageElement) {
                  var isNil3 = false;
                  var nilAttribute3 = js2xml.getAttribute(responseDoc, messageElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute3) {
                    isNil3 = nilAttribute3 === 'true';
                  }
                  if (isNil3 === false) {
                    var messageInstance = messageElement;
                    errorInstance.message = messageInstance;
                  }
                }
                
                var extendedCodeElement = js2xml.getElement(responseDoc, errorsElement, 'EXTENDEDCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (extendedCodeElement) {
                  var isNil4 = false;
                  var nilAttribute4 = js2xml.getAttribute(responseDoc, extendedCodeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute4) {
                    isNil4 = nilAttribute4 === 'true';
                  }
                  if (isNil4 === false) {
                    var extendedCodeInstance = extendedCodeElement;
                    errorInstance.extendedCode = extendedCodeInstance;
                  }
                }
                
                var messageTemplateElement = js2xml.getElement(responseDoc, errorsElement, 'MESSAGETEMPLATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (messageTemplateElement) {
                  var isNil5 = false;
                  var nilAttribute5 = js2xml.getAttribute(responseDoc, messageTemplateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute5) {
                    isNil5 = nilAttribute5 === 'true';
                  }
                  if (isNil5 === false) {
                    var messageTemplateInstance = messageTemplateElement;
                    errorInstance.messageTemplate = messageTemplateInstance;
                  }
                }
                
                var parametersSequenceElement = js2xml.getElement(responseDoc, errorsElement, 'PARAMETERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (parametersSequenceElement) {
                  var isNil6 = false;
                  var nilAttribute6 = js2xml.getAttribute(responseDoc, parametersSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute6) {
                    isNil6 = nilAttribute6 === 'true';
                  }
                  if (isNil6 === false) {
                    for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, parametersSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex2 = loweredIndex2 + 1) {
                      var parametersElement = js2xml.getElements(responseDoc, parametersSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex2];
                      errorInstance.parameters.push(parametersElement);
                    }
                  }
                }
                
                var innerErrorsElement = js2xml.getElement(responseDoc, errorsElement, 'INNERERRORS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (innerErrorsElement) {
                  var isNil7 = false;
                  var nilAttribute7 = js2xml.getAttribute(responseDoc, innerErrorsElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute7) {
                    isNil7 = nilAttribute7 === 'true';
                  }
                  if (isNil7 === false) {
                    var innerErrorsInstance = innerErrorsElement;
                    errorInstance.innerErrors = innerErrorsInstance;
                  }
                }
              }
            }
          }
          
          var expirationTimeElement = js2xml.getElement(responseDoc, operationElement, 'EXPIRATIONTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (expirationTimeElement) {
            var expirationTimeInstance = new Date(expirationTimeElement);
            result.expirationTime = expirationTimeInstance;
          }
          
          var geoMasterOperationIdElement = js2xml.getElement(responseDoc, operationElement, 'GEOMASTEROPERATIONID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (geoMasterOperationIdElement) {
            var isNil8 = false;
            var nilAttribute8 = js2xml.getAttribute(responseDoc, geoMasterOperationIdElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute8) {
              isNil8 = nilAttribute8 === 'true';
            }
            if (isNil8 === false) {
              var geoMasterOperationIdInstance = geoMasterOperationIdElement;
              result.geoMasterOperationId = geoMasterOperationIdInstance;
            }
          }
          
          var idElement = js2xml.getElement(responseDoc, operationElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var isNil9 = false;
            var nilAttribute9 = js2xml.getAttribute(responseDoc, idElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute9) {
              isNil9 = nilAttribute9 === 'true';
            }
            if (isNil9 === false) {
              var idInstance = idElement;
              result.operationId = idInstance;
            }
          }
          
          var modifiedTimeElement = js2xml.getElement(responseDoc, operationElement, 'MODIFIEDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (modifiedTimeElement) {
            var modifiedTimeInstance = new Date(modifiedTimeElement);
            result.modifiedTime = modifiedTimeInstance;
          }
          
          var nameElement = js2xml.getElement(responseDoc, operationElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var isNil10 = false;
            var nilAttribute10 = js2xml.getAttribute(responseDoc, nameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute10) {
              isNil10 = nilAttribute10 === 'true';
            }
            if (isNil10 === false) {
              var nameInstance = nameElement;
              result.name = nameInstance;
            }
          }
          
          var statusElement = js2xml.getElement(responseDoc, operationElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement) {
            var statusInstance = statusElement;
            result.status = statusInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Register your subscription to use Azure Web Sites.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteManagementClient.prototype.registerSubscription = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.baseUri;
    var url2 = '/' + (this.credentials.subscriptionId ? this.credentials.subscriptionId.trim() : '') + '/services?';
    url2 = url2 + 'service=website';
    url2 = url2 + '&action=register';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Unregister your subscription to use Azure Web Sites.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteManagementClient.prototype.unregisterSubscription = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.baseUri;
    var url2 = '/' + (this.credentials.subscriptionId ? this.credentials.subscriptionId.trim() : '') + '/services?';
    url2 = url2 + 'service=website';
    url2 = url2 + '&action=unregister';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return WebSiteManagementClient;
})();
exports.WebSiteManagementClient = WebSiteManagementClient;

var ServerFarmOperations = ( /** @lends ServerFarmOperations */ function() {
  /**
   * @class
   * Operations for managing the server farm in a web space.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [WebSiteManagementClient] {@link
   * WebSiteManagementClient~WebSiteManagementClient}.
   * See [serverFarms] {@link
   * WebSiteManagementClient~WebSiteManagementClient#serverFarms}.
   * Initializes a new instance of the ServerFarmOperations class.
   * @constructor
   * 
   * @param {WebSiteManagementClient} client Reference to the service client.
   */
  function ServerFarmOperations(client) {
    this.client = client;
  }
  
  /**
   * You can create a server farm by issuing an HTTP POST request. Only one
   * server farm per webspace is permitted. You can retrieve server farm
   * details by using HTTP GET, change server farm properties by using HTTP
   * PUT, and delete a server farm by using HTTP DELETE. A request body is
   * required for server farm creation (HTTP POST) and server farm update
   * (HTTP PUT). Warning: Creating a server farm changes your webspace's
   * Compute Mode from Shared to Dedicated. You will be charged from the
   * moment the server farm is created, even if all your sites are still
   * running in Free mode.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {ServerFarmCreateParameters} parameters Parameters supplied to the
   * Create Server Farm operation.
   * 
   * @param {number} [parameters.currentNumberOfWorkers] The current number of
   * Virtual Machines (VMs) in the server farm.
   * 
   * @param {ServerFarmWorkerSize} [parameters.currentWorkerSize] The current
   * worker size. Possible values are Small, Medium, or Large.
   * 
   * @param {number} parameters.numberOfWorkers The instance count, which is
   * the number of virtual machines dedicated to the farm. Supported values
   * are 1-10.
   * 
   * @param {ServerFarmWorkerSize} parameters.workerSize The instance size.
   * Possible values are Small, Medium, or Large.
   * 
   * @param {ServerFarmStatus} [parameters.status] The server farm status.
   * Possible values are Ready or Pending.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerFarmOperations.prototype.create = function(webSpaceName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.numberOfWorkers === null || parameters.numberOfWorkers === undefined) {
      return callback(new Error('parameters.numberOfWorkers cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/ServerFarms';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serverFarmElement = js2xml.createElement('ServerFarm', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serverFarmElement);
    
    if (parameters.currentNumberOfWorkers) {
      var currentNumberOfWorkersElement = js2xml.createElement('CurrentNumberOfWorkers', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(currentNumberOfWorkersElement, parameters.currentNumberOfWorkers.toString());
      js2xml.addChildElement(serverFarmElement, currentNumberOfWorkersElement);
    }
    
    if (parameters.currentWorkerSize) {
      var currentWorkerSizeElement = js2xml.createElement('CurrentWorkerSize', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(currentWorkerSizeElement, parameters.currentWorkerSize ? parameters.currentWorkerSize.toString() : 'Small');
      js2xml.addChildElement(serverFarmElement, currentWorkerSizeElement);
    }
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, 'DefaultServerFarm');
    js2xml.addChildElement(serverFarmElement, nameElement);
    
    var numberOfWorkersElement = js2xml.createElement('NumberOfWorkers', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(numberOfWorkersElement, parameters.numberOfWorkers.toString());
    js2xml.addChildElement(serverFarmElement, numberOfWorkersElement);
    
    var workerSizeElement = js2xml.createElement('WorkerSize', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(workerSizeElement, parameters.workerSize ? parameters.workerSize.toString() : 'Small');
    js2xml.addChildElement(serverFarmElement, workerSizeElement);
    
    if (parameters.status) {
      var statusElement = js2xml.createElement('Status', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(statusElement, parameters.status ? parameters.status.toString() : 'Ready');
      js2xml.addChildElement(serverFarmElement, statusElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverFarmElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serverFarmElement2) {
          var currentNumberOfWorkersElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentNumberOfWorkersElement2) {
            var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement2, 10);
            result.currentNumberOfWorkers = currentNumberOfWorkersInstance;
          }
          
          var currentWorkerSizeElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentWorkerSizeElement2) {
            var currentWorkerSizeInstance = currentWorkerSizeElement2;
            result.currentWorkerSize = currentWorkerSizeInstance;
          }
          
          var nameElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var numberOfWorkersElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'NUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (numberOfWorkersElement2) {
            var numberOfWorkersInstance = parseInt(numberOfWorkersElement2, 10);
            result.numberOfWorkers = numberOfWorkersInstance;
          }
          
          var workerSizeElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (workerSizeElement2) {
            var workerSizeInstance = workerSizeElement2;
            result.workerSize = workerSizeInstance;
          }
          
          var statusElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement2) {
            var statusInstance = statusElement2;
            result.status = statusInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can create a server farm by issuing an HTTP POST request. Only one
   * server farm per webspace is permitted. You can retrieve server farm
   * details by using HTTP GET, change server farm properties by using HTTP
   * PUT, and delete a server farm by using HTTP DELETE. A request body is
   * required for server farm creation (HTTP POST) and server farm update
   * (HTTP PUT). Warning: Creating a server farm changes your webspace's
   * Compute Mode from Shared to Dedicated. You will be charged from the
   * moment the server farm is created, even if all your sites are still
   * running in Free mode.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerFarmOperations.prototype.delete = function(webSpaceName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/ServerFarms/DefaultServerFarm';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * You can create a server farm by issuing an HTTP POST request. Only one
   * server farm per webspace is permitted. You can retrieve server farm
   * details by using HTTP GET, change server farm properties by using HTTP
   * PUT, and delete a server farm by using HTTP DELETE. A request body is
   * required for server farm creation (HTTP POST) and server farm update
   * (HTTP PUT). Warning: Creating a server farm changes your webspace's
   * Compute Mode from Shared to Dedicated. You will be charged from the
   * moment the server farm is created, even if all your sites are still
   * running in Free mode.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} serverFarmName The name of the server farm.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerFarmOperations.prototype.get = function(webSpaceName, serverFarmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (serverFarmName === null || serverFarmName === undefined) {
      return callback(new Error('serverFarmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/ServerFarms/' + serverFarmName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverFarmElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serverFarmElement) {
          var currentNumberOfWorkersElement = js2xml.getElement(responseDoc, serverFarmElement, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentNumberOfWorkersElement) {
            var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement, 10);
            result.currentNumberOfWorkers = currentNumberOfWorkersInstance;
          }
          
          var currentWorkerSizeElement = js2xml.getElement(responseDoc, serverFarmElement, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentWorkerSizeElement) {
            var currentWorkerSizeInstance = currentWorkerSizeElement;
            result.currentWorkerSize = currentWorkerSizeInstance;
          }
          
          var nameElement = js2xml.getElement(responseDoc, serverFarmElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            result.name = nameInstance;
          }
          
          var numberOfWorkersElement = js2xml.getElement(responseDoc, serverFarmElement, 'NUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (numberOfWorkersElement) {
            var numberOfWorkersInstance = parseInt(numberOfWorkersElement, 10);
            result.numberOfWorkers = numberOfWorkersInstance;
          }
          
          var workerSizeElement = js2xml.getElement(responseDoc, serverFarmElement, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (workerSizeElement) {
            var workerSizeInstance = workerSizeElement;
            result.workerSize = workerSizeInstance;
          }
          
          var statusElement = js2xml.getElement(responseDoc, serverFarmElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement) {
            var statusInstance = statusElement;
            result.status = statusInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can create a server farm by issuing an HTTP POST request. Only one
   * server farm per webspace is permitted. You can retrieve server farm
   * details by using HTTP GET, change server farm properties by using HTTP
   * PUT, and delete a server farm by using HTTP DELETE. A request body is
   * required for server farm creation (HTTP POST) and server farm update
   * (HTTP PUT). Warning: Creating a server farm changes your webspace's
   * Compute Mode from Shared to Dedicated. You will be charged from the
   * moment the server farm is created, even if all your sites are still
   * running in Free mode.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerFarmOperations.prototype.list = function(webSpaceName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/ServerFarms';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { serverFarms: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverFarmsSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERFARMS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serverFarmsSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serverFarmsSequenceElement, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serverFarmsElement = js2xml.getElements(responseDoc, serverFarmsSequenceElement, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serverFarmInstance = {};
            result.serverFarms.push(serverFarmInstance);
            
            var currentNumberOfWorkersElement = js2xml.getElement(responseDoc, serverFarmsElement, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (currentNumberOfWorkersElement) {
              var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement, 10);
              serverFarmInstance.currentNumberOfWorkers = currentNumberOfWorkersInstance;
            }
            
            var currentWorkerSizeElement = js2xml.getElement(responseDoc, serverFarmsElement, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (currentWorkerSizeElement) {
              var currentWorkerSizeInstance = currentWorkerSizeElement;
              serverFarmInstance.currentWorkerSize = currentWorkerSizeInstance;
            }
            
            var nameElement = js2xml.getElement(responseDoc, serverFarmsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serverFarmInstance.name = nameInstance;
            }
            
            var numberOfWorkersElement = js2xml.getElement(responseDoc, serverFarmsElement, 'NUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (numberOfWorkersElement) {
              var numberOfWorkersInstance = parseInt(numberOfWorkersElement, 10);
              serverFarmInstance.numberOfWorkers = numberOfWorkersInstance;
            }
            
            var workerSizeElement = js2xml.getElement(responseDoc, serverFarmsElement, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (workerSizeElement) {
              var workerSizeInstance = workerSizeElement;
              serverFarmInstance.workerSize = workerSizeInstance;
            }
            
            var statusElement = js2xml.getElement(responseDoc, serverFarmsElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (statusElement) {
              var statusInstance = statusElement;
              serverFarmInstance.status = statusInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can create a server farm by issuing an HTTP POST request. Only one
   * server farm per webspace is permitted. You can retrieve server farm
   * details by using HTTP GET, change server farm properties by using HTTP
   * PUT, and delete a server farm by using HTTP DELETE. A request body is
   * required for server farm creation (HTTP POST) and server farm update
   * (HTTP PUT). Warning: Creating a server farm changes your webspace's
   * Compute Mode from Shared to Dedicated. You will be charged from the
   * moment the server farm is created, even if all your sites are still
   * running in Free mode.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn194277.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {ServerFarmUpdateParameters} parameters Parameters supplied to the
   * Update Server Farm operation.
   * 
   * @param {number} [parameters.currentNumberOfWorkers] The current number of
   * Virtual Machines (VMs) in the server farm.
   * 
   * @param {ServerFarmWorkerSize} [parameters.currentWorkerSize] The current
   * worker size. Possible values are Small, Medium, or Large.
   * 
   * @param {number} parameters.numberOfWorkers The instance count, which is
   * the number of virtual machines dedicated to the farm. Supported values
   * are 1-10.
   * 
   * @param {ServerFarmWorkerSize} parameters.workerSize The instance size.
   * Possible values are Small, Medium, or Large.
   * 
   * @param {ServerFarmStatus} [parameters.status] The server farm status.
   * Possible values are Ready or Pending.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerFarmOperations.prototype.update = function(webSpaceName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.numberOfWorkers === null || parameters.numberOfWorkers === undefined) {
      return callback(new Error('parameters.numberOfWorkers cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/ServerFarms/DefaultServerFarm';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serverFarmElement = js2xml.createElement('ServerFarm', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serverFarmElement);
    
    if (parameters.currentNumberOfWorkers) {
      var currentNumberOfWorkersElement = js2xml.createElement('CurrentNumberOfWorkers', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(currentNumberOfWorkersElement, parameters.currentNumberOfWorkers.toString());
      js2xml.addChildElement(serverFarmElement, currentNumberOfWorkersElement);
    }
    
    if (parameters.currentWorkerSize) {
      var currentWorkerSizeElement = js2xml.createElement('CurrentWorkerSize', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(currentWorkerSizeElement, parameters.currentWorkerSize ? parameters.currentWorkerSize.toString() : 'Small');
      js2xml.addChildElement(serverFarmElement, currentWorkerSizeElement);
    }
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, 'DefaultServerFarm');
    js2xml.addChildElement(serverFarmElement, nameElement);
    
    var numberOfWorkersElement = js2xml.createElement('NumberOfWorkers', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(numberOfWorkersElement, parameters.numberOfWorkers.toString());
    js2xml.addChildElement(serverFarmElement, numberOfWorkersElement);
    
    var workerSizeElement = js2xml.createElement('WorkerSize', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(workerSizeElement, parameters.workerSize ? parameters.workerSize.toString() : 'Small');
    js2xml.addChildElement(serverFarmElement, workerSizeElement);
    
    if (parameters.status) {
      var statusElement = js2xml.createElement('Status', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(statusElement, parameters.status ? parameters.status.toString() : 'Ready');
      js2xml.addChildElement(serverFarmElement, statusElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverFarmElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serverFarmElement2) {
          var currentNumberOfWorkersElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentNumberOfWorkersElement2) {
            var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement2, 10);
            result.currentNumberOfWorkers = currentNumberOfWorkersInstance;
          }
          
          var currentWorkerSizeElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentWorkerSizeElement2) {
            var currentWorkerSizeInstance = currentWorkerSizeElement2;
            result.currentWorkerSize = currentWorkerSizeInstance;
          }
          
          var nameElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var numberOfWorkersElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'NUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (numberOfWorkersElement2) {
            var numberOfWorkersInstance = parseInt(numberOfWorkersElement2, 10);
            result.numberOfWorkers = numberOfWorkersInstance;
          }
          
          var workerSizeElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (workerSizeElement2) {
            var workerSizeInstance = workerSizeElement2;
            result.workerSize = workerSizeInstance;
          }
          
          var statusElement2 = js2xml.getElement(responseDoc, serverFarmElement2, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement2) {
            var statusInstance = statusElement2;
            result.status = statusInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return ServerFarmOperations;
})();

var WebSiteOperations = ( /** @lends WebSiteOperations */ function() {
  /**
   * @class
   * Operations for managing the web sites in a web space.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
   * more information)
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [WebSiteManagementClient] {@link
   * WebSiteManagementClient~WebSiteManagementClient}.
   * See [webSites] {@link
   * WebSiteManagementClient~WebSiteManagementClient#webSites}.
   * Initializes a new instance of the WebSiteOperations class.
   * @constructor
   * 
   * @param {WebSiteManagementClient} client Reference to the service client.
   */
  function WebSiteOperations(client) {
    this.client = client;
  }
  
  /**
   * You can swap a web site from one slot to another slot.
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {string} sourceSlotName The name of the first web site slot to swap
   * (source).
   * 
   * @param {string} targetSlotName The name of the second web site slot to
   * swap with (target).
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.beginSwapingSlots = function(webSpaceName, webSiteName, sourceSlotName, targetSlotName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    if (sourceSlotName === null || sourceSlotName === undefined) {
      return callback(new Error('sourceSlotName cannot be null.'));
    }
    if (targetSlotName === null || targetSlotName === undefined) {
      return callback(new Error('targetSlotName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '(' + sourceSlotName.trim() + ')/slots?';
    url2 = url2 + 'Command=swap';
    url2 = url2 + '&targetSlot=' + encodeURIComponent(targetSlotName.trim());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Length'] = '0';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { errors: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var operationElement = js2xml.getElement(responseDoc, responseDoc, 'OPERATION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (operationElement) {
          var createdTimeElement = js2xml.getElement(responseDoc, operationElement, 'CREATEDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (createdTimeElement) {
            var createdTimeInstance = new Date(createdTimeElement);
            result.createdTime = createdTimeInstance;
          }
          
          var errorsSequenceElement = js2xml.getElement(responseDoc, operationElement, 'ERRORS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorsSequenceElement) {
            var isNil = false;
            var nilAttribute = js2xml.getAttribute(responseDoc, errorsSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute) {
              isNil = nilAttribute === 'true';
            }
            if (isNil === false) {
              for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, errorsSequenceElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
                var errorsElement = js2xml.getElements(responseDoc, errorsSequenceElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
                var errorInstance = { parameters: [] };
                result.errors.push(errorInstance);
                
                var codeElement = js2xml.getElement(responseDoc, errorsElement, 'CODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (codeElement) {
                  var isNil2 = false;
                  var nilAttribute2 = js2xml.getAttribute(responseDoc, codeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute2) {
                    isNil2 = nilAttribute2 === 'true';
                  }
                  if (isNil2 === false) {
                    var codeInstance = codeElement;
                    errorInstance.code = codeInstance;
                  }
                }
                
                var messageElement = js2xml.getElement(responseDoc, errorsElement, 'MESSAGE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (messageElement) {
                  var isNil3 = false;
                  var nilAttribute3 = js2xml.getAttribute(responseDoc, messageElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute3) {
                    isNil3 = nilAttribute3 === 'true';
                  }
                  if (isNil3 === false) {
                    var messageInstance = messageElement;
                    errorInstance.message = messageInstance;
                  }
                }
                
                var extendedCodeElement = js2xml.getElement(responseDoc, errorsElement, 'EXTENDEDCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (extendedCodeElement) {
                  var isNil4 = false;
                  var nilAttribute4 = js2xml.getAttribute(responseDoc, extendedCodeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute4) {
                    isNil4 = nilAttribute4 === 'true';
                  }
                  if (isNil4 === false) {
                    var extendedCodeInstance = extendedCodeElement;
                    errorInstance.extendedCode = extendedCodeInstance;
                  }
                }
                
                var messageTemplateElement = js2xml.getElement(responseDoc, errorsElement, 'MESSAGETEMPLATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (messageTemplateElement) {
                  var isNil5 = false;
                  var nilAttribute5 = js2xml.getAttribute(responseDoc, messageTemplateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute5) {
                    isNil5 = nilAttribute5 === 'true';
                  }
                  if (isNil5 === false) {
                    var messageTemplateInstance = messageTemplateElement;
                    errorInstance.messageTemplate = messageTemplateInstance;
                  }
                }
                
                var parametersSequenceElement = js2xml.getElement(responseDoc, errorsElement, 'PARAMETERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (parametersSequenceElement) {
                  var isNil6 = false;
                  var nilAttribute6 = js2xml.getAttribute(responseDoc, parametersSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute6) {
                    isNil6 = nilAttribute6 === 'true';
                  }
                  if (isNil6 === false) {
                    for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, parametersSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex2 = loweredIndex2 + 1) {
                      var parametersElement = js2xml.getElements(responseDoc, parametersSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex2];
                      errorInstance.parameters.push(parametersElement);
                    }
                  }
                }
                
                var innerErrorsElement = js2xml.getElement(responseDoc, errorsElement, 'INNERERRORS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (innerErrorsElement) {
                  var isNil7 = false;
                  var nilAttribute7 = js2xml.getAttribute(responseDoc, innerErrorsElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute7) {
                    isNil7 = nilAttribute7 === 'true';
                  }
                  if (isNil7 === false) {
                    var innerErrorsInstance = innerErrorsElement;
                    errorInstance.innerErrors = innerErrorsInstance;
                  }
                }
              }
            }
          }
          
          var expirationTimeElement = js2xml.getElement(responseDoc, operationElement, 'EXPIRATIONTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (expirationTimeElement) {
            var expirationTimeInstance = new Date(expirationTimeElement);
            result.expirationTime = expirationTimeInstance;
          }
          
          var geoMasterOperationIdElement = js2xml.getElement(responseDoc, operationElement, 'GEOMASTEROPERATIONID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (geoMasterOperationIdElement) {
            var isNil8 = false;
            var nilAttribute8 = js2xml.getAttribute(responseDoc, geoMasterOperationIdElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute8) {
              isNil8 = nilAttribute8 === 'true';
            }
            if (isNil8 === false) {
              var geoMasterOperationIdInstance = geoMasterOperationIdElement;
              result.geoMasterOperationId = geoMasterOperationIdInstance;
            }
          }
          
          var idElement = js2xml.getElement(responseDoc, operationElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var isNil9 = false;
            var nilAttribute9 = js2xml.getAttribute(responseDoc, idElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute9) {
              isNil9 = nilAttribute9 === 'true';
            }
            if (isNil9 === false) {
              var idInstance = idElement;
              result.operationId = idInstance;
            }
          }
          
          var modifiedTimeElement = js2xml.getElement(responseDoc, operationElement, 'MODIFIEDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (modifiedTimeElement) {
            var modifiedTimeInstance = new Date(modifiedTimeElement);
            result.modifiedTime = modifiedTimeInstance;
          }
          
          var nameElement = js2xml.getElement(responseDoc, operationElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var isNil10 = false;
            var nilAttribute10 = js2xml.getAttribute(responseDoc, nameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute10) {
              isNil10 = nilAttribute10 === 'true';
            }
            if (isNil10 === false) {
              var nameInstance = nameElement;
              result.name = nameInstance;
            }
          }
          
          var statusElement = js2xml.getElement(responseDoc, operationElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement) {
            var statusInstance = statusElement;
            result.status = statusInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can create a web site by using a POST request that includes the name
   * of the web site and other information in the request body.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166986.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {WebSiteCreateParameters} parameters Parameters supplied to the
   * Create Web Site operation.
   * 
   * @param {array} [parameters.hostNames] The fully qualified domain name for
   * web site. Only one host name can be specified in the azurewebsites.net
   * domain. The host name should match the name of the web site. Custom
   * domains can only be specified for Shared or Standard web sites.
   * 
   * @param {string} parameters.name The name of the web site. This should
   * match the host name in the default domain.
   * 
   * @param {WebSiteComputeMode} [parameters.computeMode] The compute mode for
   * the web site. This value should be Shared for the Free or Paid Shared
   * offerings or Dedicated for the Standard offering. The default value is
   * Shared. If you set ComputeMode to Dedicated, you must specify a value for
   * the ServerFarm element.
   * 
   * @param {string} [parameters.serverFarm] The name of the Server Farm
   * associated with this web site. This is a required value for Standard mode.
   * 
   * @param {WebSiteMode} [parameters.siteMode] The web site mode. This value
   * is Limited for the Free offering and Basic for the Paid and Shared
   * offerings. The default value is Limited. Note: Standard mode does not use
   * the SiteMode setting; it uses the ComputeMode setting. For more
   * information, see Upgrade or Downgrade a Web Site.
   * 
   * @param {string} parameters.webSpaceName The name of the web space.
   * 
   * @param {WebSpaceDetails} [parameters.webSpace] Information about the web
   * space to create.
   * 
   * @param {string} [parameters.webSpace.geoRegion] The geographical region of
   * the web space that will be created.
   * 
   * @param {string} [parameters.webSpace.name] The name of the web space.
   * 
   * @param {string} [parameters.webSpace.plan] The web space plan. This value
   * must be VirtualDedicatedPlan.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.create = function(webSpaceName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    if (parameters.webSpace) {
      if (parameters.webSpace.geoRegion === null || parameters.webSpace.geoRegion === undefined) {
        return callback(new Error('parameters.webSpace.geoRegion cannot be null.'));
      }
      if (parameters.webSpace.name === null || parameters.webSpace.name === undefined) {
        return callback(new Error('parameters.webSpace.name cannot be null.'));
      }
      if (parameters.webSpace.plan === null || parameters.webSpace.plan === undefined) {
        return callback(new Error('parameters.webSpace.plan cannot be null.'));
      }
    }
    if (parameters.webSpaceName === null || parameters.webSpaceName === undefined) {
      return callback(new Error('parameters.webSpaceName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var siteElement = js2xml.createElement('Site', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, siteElement);
    
    if (parameters.hostNames) {
      var hostNamesSequenceElement = js2xml.createElement('HostNames', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex1 = 0; loweredIndex1 < parameters.hostNames.length; loweredIndex1 = loweredIndex1 + 1) {
        var hostNamesItem = parameters.hostNames[loweredIndex1];
        var hostNamesItemElement = js2xml.createElement('string', 'http://schemas.microsoft.com/2003/10/Serialization/Arrays');
        js2xml.setElementValue(hostNamesItemElement, hostNamesItem);
        js2xml.addChildElement(hostNamesSequenceElement, hostNamesItemElement);
      }
      js2xml.addChildElement(siteElement, hostNamesSequenceElement);
    }
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(siteElement, nameElement);
    
    if (parameters.computeMode) {
      var computeModeElement = js2xml.createElement('ComputeMode', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(computeModeElement, parameters.computeMode ? parameters.computeMode.toString() : 'Shared');
      js2xml.addChildElement(siteElement, computeModeElement);
    }
    
    if (parameters.serverFarm) {
      var serverFarmElement = js2xml.createElement('ServerFarm', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serverFarmElement, parameters.serverFarm);
      js2xml.addChildElement(siteElement, serverFarmElement);
    }
    
    if (parameters.siteMode) {
      var siteModeElement = js2xml.createElement('SiteMode', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(siteModeElement, parameters.siteMode ? parameters.siteMode.toString() : 'Limited');
      js2xml.addChildElement(siteElement, siteModeElement);
    }
    
    var webSpaceElement = js2xml.createElement('WebSpace', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(webSpaceElement, parameters.webSpaceName);
    js2xml.addChildElement(siteElement, webSpaceElement);
    
    if (parameters.webSpace) {
      var webSpaceToCreateElement = js2xml.createElement('WebSpaceToCreate', 'http://schemas.microsoft.com/windowsazure');
      js2xml.addChildElement(siteElement, webSpaceToCreateElement);
      
      var geoRegionElement = js2xml.createElement('GeoRegion', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(geoRegionElement, parameters.webSpace.geoRegion);
      js2xml.addChildElement(webSpaceToCreateElement, geoRegionElement);
      
      var nameElement2 = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(nameElement2, parameters.webSpace.name);
      js2xml.addChildElement(webSpaceToCreateElement, nameElement2);
      
      var planElement = js2xml.createElement('Plan', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(planElement, parameters.webSpace.plan);
      js2xml.addChildElement(webSpaceToCreateElement, planElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var siteElement2 = js2xml.getElement(responseDoc, responseDoc, 'SITE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (siteElement2) {
          var webSiteInstance = { enabledHostNames: [], hostNames: [], hostNameSslStates: [], sslCertificates: [] };
          result.webSite = webSiteInstance;
          
          var adminEnabledElement = js2xml.getElement(responseDoc, siteElement2, 'ADMINENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (adminEnabledElement) {
            var adminEnabledInstance = adminEnabledElement === 'true';
            webSiteInstance.adminEnabled = adminEnabledInstance;
          }
          
          var availabilityStateElement = js2xml.getElement(responseDoc, siteElement2, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (availabilityStateElement) {
            var availabilityStateInstance = availabilityStateElement;
            webSiteInstance.availabilityState = availabilityStateInstance;
          }
          
          var computeModeElement2 = js2xml.getElement(responseDoc, siteElement2, 'COMPUTEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (computeModeElement2) {
            var computeModeInstance = computeModeElement2;
            webSiteInstance.computeMode = computeModeInstance;
          }
          
          var enabledElement = js2xml.getElement(responseDoc, siteElement2, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledElement) {
            var enabledInstance = enabledElement === 'true';
            webSiteInstance.enabled = enabledInstance;
          }
          
          var enabledHostNamesSequenceElement = js2xml.getElement(responseDoc, siteElement2, 'ENABLEDHOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledHostNamesSequenceElement) {
            for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex2 = loweredIndex2 + 1) {
              var enabledHostNamesElement = js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex2];
              webSiteInstance.enabledHostNames.push(enabledHostNamesElement);
            }
          }
          
          var hostNameSslStatesSequenceElement = js2xml.getElement(responseDoc, siteElement2, 'HOSTNAMESSLSTATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNameSslStatesSequenceElement) {
            for (var loweredIndex3 = 0; loweredIndex3 < js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex3 = loweredIndex3 + 1) {
              var hostNameSslStatesElement = js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex3];
              var webSiteHostNameSslStateInstance = {};
              webSiteInstance.hostNameSslStates.push(webSiteHostNameSslStateInstance);
              
              var nameElement3 = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement3) {
                var nameInstance = nameElement3;
                webSiteHostNameSslStateInstance.name = nameInstance;
              }
              
              var sslStateElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'SSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (sslStateElement) {
                var sslStateInstance = sslStateElement;
                webSiteHostNameSslStateInstance.sslState = sslStateInstance;
              }
              
              var thumbprintElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (thumbprintElement) {
                var isNil = false;
                var nilAttribute = js2xml.getAttribute(responseDoc, thumbprintElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute) {
                  isNil = nilAttribute === 'true';
                }
                if (isNil === false) {
                  var thumbprintInstance = thumbprintElement;
                  webSiteHostNameSslStateInstance.thumbprint = thumbprintInstance;
                }
              }
              
              var virtualIPElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'VIRTUALIP', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (virtualIPElement) {
                var isNil2 = false;
                var nilAttribute2 = js2xml.getAttribute(responseDoc, virtualIPElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute2) {
                  isNil2 = nilAttribute2 === 'true';
                }
                if (isNil2 === false) {
                  var virtualIPInstance = virtualIPElement;
                  webSiteHostNameSslStateInstance.virtualIP = virtualIPInstance;
                }
              }
            }
          }
          
          var hostNamesSequenceElement2 = js2xml.getElement(responseDoc, siteElement2, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNamesSequenceElement2) {
            for (var loweredIndex4 = 0; loweredIndex4 < js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex4 = loweredIndex4 + 1) {
              var hostNamesElement = js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex4];
              webSiteInstance.hostNames.push(hostNamesElement);
            }
          }
          
          var lastModifiedTimeUtcElement = js2xml.getElement(responseDoc, siteElement2, 'LASTMODIFIEDTIMEUTC', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (lastModifiedTimeUtcElement) {
            var lastModifiedTimeUtcInstance = new Date(lastModifiedTimeUtcElement);
            webSiteInstance.lastModifiedTimeUtc = lastModifiedTimeUtcInstance;
          }
          
          var nameElement4 = js2xml.getElement(responseDoc, siteElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement4) {
            var nameInstance2 = nameElement4;
            webSiteInstance.name = nameInstance2;
          }
          
          var ownerElement = js2xml.getElement(responseDoc, siteElement2, 'OWNER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (ownerElement) {
            var isNil3 = false;
            var nilAttribute3 = js2xml.getAttribute(responseDoc, ownerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute3) {
              isNil3 = nilAttribute3 === 'true';
            }
            if (isNil3 === false) {
              var ownerInstance = ownerElement;
              webSiteInstance.owner = ownerInstance;
            }
          }
          
          var repositorySiteNameElement = js2xml.getElement(responseDoc, siteElement2, 'REPOSITORYSITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (repositorySiteNameElement) {
            var repositorySiteNameInstance = repositorySiteNameElement;
            webSiteInstance.repositorySiteName = repositorySiteNameInstance;
          }
          
          var runtimeAvailabilityStateElement = js2xml.getElement(responseDoc, siteElement2, 'RUNTIMEAVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (runtimeAvailabilityStateElement) {
            var runtimeAvailabilityStateInstance = runtimeAvailabilityStateElement;
            webSiteInstance.runtimeAvailabilityState = runtimeAvailabilityStateInstance;
          }
          
          var sSLCertificatesSequenceElement = js2xml.getElement(responseDoc, siteElement2, 'SSLCERTIFICATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sSLCertificatesSequenceElement) {
            var isNil4 = false;
            var nilAttribute4 = js2xml.getAttribute(responseDoc, sSLCertificatesSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute4) {
              isNil4 = nilAttribute4 === 'true';
            }
            if (isNil4 === false) {
              for (var loweredIndex5 = 0; loweredIndex5 < js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex5 = loweredIndex5 + 1) {
                var sSLCertificatesElement = js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex5];
                var certificateInstance = { hostNames: [] };
                webSiteInstance.sslCertificates.push(certificateInstance);
                
                var expirationDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'EXPIRATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (expirationDateElement && (expirationDateElement === null || expirationDateElement === undefined || expirationDateElement.length === 0) === false) {
                  var isNil5 = false;
                  var nilAttribute5 = js2xml.getAttribute(responseDoc, expirationDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute5) {
                    isNil5 = nilAttribute5 === 'true';
                  }
                  if (isNil5 === false) {
                    var expirationDateInstance = new Date(expirationDateElement);
                    certificateInstance.expirationDate = expirationDateInstance;
                  }
                }
                
                var friendlyNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'FRIENDLYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (friendlyNameElement) {
                  var isNil6 = false;
                  var nilAttribute6 = js2xml.getAttribute(responseDoc, friendlyNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute6) {
                    isNil6 = nilAttribute6 === 'true';
                  }
                  if (isNil6 === false) {
                    var friendlyNameInstance = friendlyNameElement;
                    certificateInstance.friendlyName = friendlyNameInstance;
                  }
                }
                
                var hostNamesSequenceElement3 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (hostNamesSequenceElement3) {
                  var isNil7 = false;
                  var nilAttribute7 = js2xml.getAttribute(responseDoc, hostNamesSequenceElement3, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute7) {
                    isNil7 = nilAttribute7 === 'true';
                  }
                  if (isNil7 === false) {
                    for (var loweredIndex6 = 0; loweredIndex6 < js2xml.getElements(responseDoc, hostNamesSequenceElement3, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex6 = loweredIndex6 + 1) {
                      var hostNamesElement2 = js2xml.getElements(responseDoc, hostNamesSequenceElement3, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex6];
                      certificateInstance.hostNames.push(hostNamesElement2);
                    }
                  }
                }
                
                var issueDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUEDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issueDateElement && (issueDateElement === null || issueDateElement === undefined || issueDateElement.length === 0) === false) {
                  var isNil8 = false;
                  var nilAttribute8 = js2xml.getAttribute(responseDoc, issueDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute8) {
                    isNil8 = nilAttribute8 === 'true';
                  }
                  if (isNil8 === false) {
                    var issueDateInstance = new Date(issueDateElement);
                    certificateInstance.issueDate = issueDateInstance;
                  }
                }
                
                var issuerElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issuerElement) {
                  var isNil9 = false;
                  var nilAttribute9 = js2xml.getAttribute(responseDoc, issuerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute9) {
                    isNil9 = nilAttribute9 === 'true';
                  }
                  if (isNil9 === false) {
                    var issuerInstance = issuerElement;
                    certificateInstance.issuer = issuerInstance;
                  }
                }
                
                var passwordElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (passwordElement) {
                  var isNil10 = false;
                  var nilAttribute10 = js2xml.getAttribute(responseDoc, passwordElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute10) {
                    isNil10 = nilAttribute10 === 'true';
                  }
                  if (isNil10 === false) {
                    var passwordInstance = passwordElement;
                    certificateInstance.password = passwordInstance;
                  }
                }
                
                var pfxBlobElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PFXBLOB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (pfxBlobElement) {
                  var isNil11 = false;
                  var nilAttribute11 = js2xml.getAttribute(responseDoc, pfxBlobElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute11) {
                    isNil11 = nilAttribute11 === 'true';
                  }
                  if (isNil11 === false) {
                    var pfxBlobInstance = new Buffer(pfxBlobElement, 'base64').toString('utf8');
                    certificateInstance.pfxBlob = pfxBlobInstance;
                  }
                }
                
                var selfLinkElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (selfLinkElement) {
                  var isNil12 = false;
                  var nilAttribute12 = js2xml.getAttribute(responseDoc, selfLinkElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute12) {
                    isNil12 = nilAttribute12 === 'true';
                  }
                  if (isNil12 === false) {
                    var selfLinkInstance = selfLinkElement;
                    certificateInstance.selfLinkUri = selfLinkInstance;
                  }
                }
                
                var siteNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (siteNameElement) {
                  var isNil13 = false;
                  var nilAttribute13 = js2xml.getAttribute(responseDoc, siteNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute13) {
                    isNil13 = nilAttribute13 === 'true';
                  }
                  if (isNil13 === false) {
                    var siteNameInstance = siteNameElement;
                    certificateInstance.siteName = siteNameInstance;
                  }
                }
                
                var subjectNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SUBJECTNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (subjectNameElement) {
                  var isNil14 = false;
                  var nilAttribute14 = js2xml.getAttribute(responseDoc, subjectNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute14) {
                    isNil14 = nilAttribute14 === 'true';
                  }
                  if (isNil14 === false) {
                    var subjectNameInstance = subjectNameElement;
                    certificateInstance.subjectName = subjectNameInstance;
                  }
                }
                
                var thumbprintElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (thumbprintElement2) {
                  var isNil15 = false;
                  var nilAttribute15 = js2xml.getAttribute(responseDoc, thumbprintElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute15) {
                    isNil15 = nilAttribute15 === 'true';
                  }
                  if (isNil15 === false) {
                    var thumbprintInstance2 = thumbprintElement2;
                    certificateInstance.thumbprint = thumbprintInstance2;
                  }
                }
                
                var toDeleteElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'TODELETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (toDeleteElement && (toDeleteElement === null || toDeleteElement === undefined || toDeleteElement.length === 0) === false) {
                  var isNil16 = false;
                  var nilAttribute16 = js2xml.getAttribute(responseDoc, toDeleteElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute16) {
                    isNil16 = nilAttribute16 === 'true';
                  }
                  if (isNil16 === false) {
                    var toDeleteInstance = toDeleteElement === 'true';
                    certificateInstance.isToBeDeleted = toDeleteInstance;
                  }
                }
                
                var validElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'VALID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (validElement && (validElement === null || validElement === undefined || validElement.length === 0) === false) {
                  var isNil17 = false;
                  var nilAttribute17 = js2xml.getAttribute(responseDoc, validElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute17) {
                    isNil17 = nilAttribute17 === 'true';
                  }
                  if (isNil17 === false) {
                    var validInstance = validElement === 'true';
                    certificateInstance.isValid = validInstance;
                  }
                }
              }
            }
          }
          
          var selfLinkElement2 = js2xml.getElement(responseDoc, siteElement2, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (selfLinkElement2) {
            var selfLinkInstance2 = selfLinkElement2;
            webSiteInstance.uri = selfLinkInstance2;
          }
          
          var serverFarmElement2 = js2xml.getElement(responseDoc, siteElement2, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serverFarmElement2) {
            var serverFarmInstance = serverFarmElement2;
            webSiteInstance.serverFarm = serverFarmInstance;
          }
          
          var siteModeElement2 = js2xml.getElement(responseDoc, siteElement2, 'SITEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (siteModeElement2) {
            var siteModeInstance = siteModeElement2;
            webSiteInstance.siteMode = siteModeInstance;
          }
          
          var sitePropertiesElement = js2xml.getElement(responseDoc, siteElement2, 'SITEPROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sitePropertiesElement) {
            var sitePropertiesInstance = { appSettings: {}, metadata: {}, properties: {} };
            webSiteInstance.siteProperties = sitePropertiesInstance;
            
            var appSettingsSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'APPSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (appSettingsSequenceElement) {
              for (var loweredIndex7 = 0; loweredIndex7 < js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex7 = loweredIndex7 + 1) {
                var appSettingsElement = js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex7];
                var appSettingsKey = js2xml.getElement(responseDoc, appSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var appSettingsValue = js2xml.getElement(responseDoc, appSettingsElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.appSettings[appSettingsKey] = appSettingsValue;
              }
            }
            
            var metadataSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'METADATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (metadataSequenceElement) {
              for (var loweredIndex8 = 0; loweredIndex8 < js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex8 = loweredIndex8 + 1) {
                var metadataElement = js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex8];
                var metadataKey = js2xml.getElement(responseDoc, metadataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var metadataValue = js2xml.getElement(responseDoc, metadataElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.metadata[metadataKey] = metadataValue;
              }
            }
            
            var propertiesSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'PROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (propertiesSequenceElement) {
              for (var loweredIndex9 = 0; loweredIndex9 < js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex9 = loweredIndex9 + 1) {
                var propertiesElement = js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex9];
                var propertiesKey = js2xml.getElement(responseDoc, propertiesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var propertiesValue = js2xml.getElement(responseDoc, propertiesElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.properties[propertiesKey] = propertiesValue;
              }
            }
          }
          
          var stateElement = js2xml.getElement(responseDoc, siteElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            webSiteInstance.state = stateInstance;
          }
          
          var usageStateElement = js2xml.getElement(responseDoc, siteElement2, 'USAGESTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (usageStateElement) {
            var usageStateInstance = usageStateElement;
            webSiteInstance.usageState = usageStateInstance;
          }
          
          var webSpaceElement2 = js2xml.getElement(responseDoc, siteElement2, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (webSpaceElement2) {
            var webSpaceInstance = webSpaceElement2;
            webSiteInstance.webSpace = webSpaceInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * A web site repository is essentially a Git repository that you can use to
   * manage your web site content. By using Git source control tools, you can
   * push or pull version-controlled changes to your site. This API executes a
   * repository create operation.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.createRepository = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/repository';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Length'] = '0';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * You can delete a web site by issuing an HTTP DELETE request. If the web
   * site being deleted is the only site remaining in a server farm, you can
   * optionally delete the server farm as well by using the
   * deleteEmptyServerFarm parameter.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn236430.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {WebSiteDeleteParameters} parameters Parameters supplied to the
   * Delete Web Site operation.
   * 
   * @param {boolean} parameters.deleteEmptyServerFarm Indicates if the server
   * farm should be deleted. You can delete the server farm if the web site
   * being deleted is the last web site in a server farm.
   * 
   * @param {boolean} parameters.deleteMetrics Indicates if the metrics for the
   * web site should be deleted.
   * 
   * @param {boolean} parameters.deleteAllSlots Indicates if all slots for the
   * web site should be deleted.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.delete = function(webSpaceName, webSiteName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.deleteAllSlots === null || parameters.deleteAllSlots === undefined) {
      return callback(new Error('parameters.deleteAllSlots cannot be null.'));
    }
    if (parameters.deleteEmptyServerFarm === null || parameters.deleteEmptyServerFarm === undefined) {
      return callback(new Error('parameters.deleteEmptyServerFarm cannot be null.'));
    }
    if (parameters.deleteMetrics === null || parameters.deleteMetrics === undefined) {
      return callback(new Error('parameters.deleteMetrics cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '?';
    url2 = url2 + 'deleteEmptyServerFarm=' + encodeURIComponent(parameters.deleteEmptyServerFarm.toString());
    url2 = url2 + '&deleteMetrics=' + encodeURIComponent(parameters.deleteMetrics.toString());
    url2 = url2 + '&deleteAllSlots=' + encodeURIComponent(parameters.deleteAllSlots.toString());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * A web site repository is essentially a Git repository that you can use to
   * manage your web site content. By using Git source control tools, you can
   * push or pull version-controlled changes to your site. This API executes a
   * repository delete operation.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.deleteRepository = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/repository';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var anyURIElement = js2xml.getElement(responseDoc, responseDoc, 'ANYURI', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (anyURIElement) {
          result.uri = anyURIElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can generate a new random password for publishing a site by issuing an
   * HTTP POST request. Tip: If you want to verify that the publish password
   * has changed, issue an HTTP GET on /publishxml before calling
   * /newpassword. In the publish XML, note the hash value in the userPWD
   * attribute. After calling /newpassword, call /publishxml again. You can
   * then compare the new value of userPWD in the Publish XML with the one you
   * noted earlier.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn236428.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.generatePassword = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/newpassword';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Length'] = '0';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * You can retrieve details for a web site by issuing an HTTP GET request.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/dn167007.aspx
   * for more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {WebSiteGetParameters} [parametersOrCallback] Parameters supplied
   * to the Get Web Site Operation.
   * 
   * @param {array} [parametersOrCallback.propertiesToInclude] Specifies a list
   * of the names of any additional parameters to return.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.get = function(webSpaceName, webSiteName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '?';
    if (parameters && parameters.propertiesToInclude && parameters.propertiesToInclude.length > 0) {
      url2 = url2 + 'propertiesToInclude=' + encodeURIComponent(parameters.propertiesToInclude.join(','));
    }
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var siteElement = js2xml.getElement(responseDoc, responseDoc, 'SITE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (siteElement) {
          var webSiteInstance = { enabledHostNames: [], hostNames: [], hostNameSslStates: [], sslCertificates: [] };
          result.webSite = webSiteInstance;
          
          var adminEnabledElement = js2xml.getElement(responseDoc, siteElement, 'ADMINENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (adminEnabledElement) {
            var adminEnabledInstance = adminEnabledElement === 'true';
            webSiteInstance.adminEnabled = adminEnabledInstance;
          }
          
          var availabilityStateElement = js2xml.getElement(responseDoc, siteElement, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (availabilityStateElement) {
            var availabilityStateInstance = availabilityStateElement;
            webSiteInstance.availabilityState = availabilityStateInstance;
          }
          
          var computeModeElement = js2xml.getElement(responseDoc, siteElement, 'COMPUTEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (computeModeElement) {
            var computeModeInstance = computeModeElement;
            webSiteInstance.computeMode = computeModeInstance;
          }
          
          var enabledElement = js2xml.getElement(responseDoc, siteElement, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledElement) {
            var enabledInstance = enabledElement === 'true';
            webSiteInstance.enabled = enabledInstance;
          }
          
          var enabledHostNamesSequenceElement = js2xml.getElement(responseDoc, siteElement, 'ENABLEDHOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledHostNamesSequenceElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex1 = loweredIndex1 + 1) {
              var enabledHostNamesElement = js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex1];
              webSiteInstance.enabledHostNames.push(enabledHostNamesElement);
            }
          }
          
          var hostNameSslStatesSequenceElement = js2xml.getElement(responseDoc, siteElement, 'HOSTNAMESSLSTATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNameSslStatesSequenceElement) {
            for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex2 = loweredIndex2 + 1) {
              var hostNameSslStatesElement = js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex2];
              var webSiteHostNameSslStateInstance = {};
              webSiteInstance.hostNameSslStates.push(webSiteHostNameSslStateInstance);
              
              var nameElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement) {
                var nameInstance = nameElement;
                webSiteHostNameSslStateInstance.name = nameInstance;
              }
              
              var sslStateElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'SSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (sslStateElement) {
                var sslStateInstance = sslStateElement;
                webSiteHostNameSslStateInstance.sslState = sslStateInstance;
              }
              
              var thumbprintElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (thumbprintElement) {
                var isNil = false;
                var nilAttribute = js2xml.getAttribute(responseDoc, thumbprintElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute) {
                  isNil = nilAttribute === 'true';
                }
                if (isNil === false) {
                  var thumbprintInstance = thumbprintElement;
                  webSiteHostNameSslStateInstance.thumbprint = thumbprintInstance;
                }
              }
              
              var virtualIPElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'VIRTUALIP', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (virtualIPElement) {
                var isNil2 = false;
                var nilAttribute2 = js2xml.getAttribute(responseDoc, virtualIPElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute2) {
                  isNil2 = nilAttribute2 === 'true';
                }
                if (isNil2 === false) {
                  var virtualIPInstance = virtualIPElement;
                  webSiteHostNameSslStateInstance.virtualIP = virtualIPInstance;
                }
              }
            }
          }
          
          var hostNamesSequenceElement = js2xml.getElement(responseDoc, siteElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNamesSequenceElement) {
            for (var loweredIndex3 = 0; loweredIndex3 < js2xml.getElements(responseDoc, hostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex3 = loweredIndex3 + 1) {
              var hostNamesElement = js2xml.getElements(responseDoc, hostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex3];
              webSiteInstance.hostNames.push(hostNamesElement);
            }
          }
          
          var lastModifiedTimeUtcElement = js2xml.getElement(responseDoc, siteElement, 'LASTMODIFIEDTIMEUTC', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (lastModifiedTimeUtcElement) {
            var lastModifiedTimeUtcInstance = new Date(lastModifiedTimeUtcElement);
            webSiteInstance.lastModifiedTimeUtc = lastModifiedTimeUtcInstance;
          }
          
          var nameElement2 = js2xml.getElement(responseDoc, siteElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance2 = nameElement2;
            webSiteInstance.name = nameInstance2;
          }
          
          var ownerElement = js2xml.getElement(responseDoc, siteElement, 'OWNER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (ownerElement) {
            var isNil3 = false;
            var nilAttribute3 = js2xml.getAttribute(responseDoc, ownerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute3) {
              isNil3 = nilAttribute3 === 'true';
            }
            if (isNil3 === false) {
              var ownerInstance = ownerElement;
              webSiteInstance.owner = ownerInstance;
            }
          }
          
          var repositorySiteNameElement = js2xml.getElement(responseDoc, siteElement, 'REPOSITORYSITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (repositorySiteNameElement) {
            var repositorySiteNameInstance = repositorySiteNameElement;
            webSiteInstance.repositorySiteName = repositorySiteNameInstance;
          }
          
          var runtimeAvailabilityStateElement = js2xml.getElement(responseDoc, siteElement, 'RUNTIMEAVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (runtimeAvailabilityStateElement) {
            var runtimeAvailabilityStateInstance = runtimeAvailabilityStateElement;
            webSiteInstance.runtimeAvailabilityState = runtimeAvailabilityStateInstance;
          }
          
          var sSLCertificatesSequenceElement = js2xml.getElement(responseDoc, siteElement, 'SSLCERTIFICATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sSLCertificatesSequenceElement) {
            var isNil4 = false;
            var nilAttribute4 = js2xml.getAttribute(responseDoc, sSLCertificatesSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute4) {
              isNil4 = nilAttribute4 === 'true';
            }
            if (isNil4 === false) {
              for (var loweredIndex4 = 0; loweredIndex4 < js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex4 = loweredIndex4 + 1) {
                var sSLCertificatesElement = js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex4];
                var certificateInstance = { hostNames: [] };
                webSiteInstance.sslCertificates.push(certificateInstance);
                
                var expirationDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'EXPIRATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (expirationDateElement && (expirationDateElement === null || expirationDateElement === undefined || expirationDateElement.length === 0) === false) {
                  var isNil5 = false;
                  var nilAttribute5 = js2xml.getAttribute(responseDoc, expirationDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute5) {
                    isNil5 = nilAttribute5 === 'true';
                  }
                  if (isNil5 === false) {
                    var expirationDateInstance = new Date(expirationDateElement);
                    certificateInstance.expirationDate = expirationDateInstance;
                  }
                }
                
                var friendlyNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'FRIENDLYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (friendlyNameElement) {
                  var isNil6 = false;
                  var nilAttribute6 = js2xml.getAttribute(responseDoc, friendlyNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute6) {
                    isNil6 = nilAttribute6 === 'true';
                  }
                  if (isNil6 === false) {
                    var friendlyNameInstance = friendlyNameElement;
                    certificateInstance.friendlyName = friendlyNameInstance;
                  }
                }
                
                var hostNamesSequenceElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (hostNamesSequenceElement2) {
                  var isNil7 = false;
                  var nilAttribute7 = js2xml.getAttribute(responseDoc, hostNamesSequenceElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute7) {
                    isNil7 = nilAttribute7 === 'true';
                  }
                  if (isNil7 === false) {
                    for (var loweredIndex5 = 0; loweredIndex5 < js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex5 = loweredIndex5 + 1) {
                      var hostNamesElement2 = js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex5];
                      certificateInstance.hostNames.push(hostNamesElement2);
                    }
                  }
                }
                
                var issueDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUEDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issueDateElement && (issueDateElement === null || issueDateElement === undefined || issueDateElement.length === 0) === false) {
                  var isNil8 = false;
                  var nilAttribute8 = js2xml.getAttribute(responseDoc, issueDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute8) {
                    isNil8 = nilAttribute8 === 'true';
                  }
                  if (isNil8 === false) {
                    var issueDateInstance = new Date(issueDateElement);
                    certificateInstance.issueDate = issueDateInstance;
                  }
                }
                
                var issuerElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issuerElement) {
                  var isNil9 = false;
                  var nilAttribute9 = js2xml.getAttribute(responseDoc, issuerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute9) {
                    isNil9 = nilAttribute9 === 'true';
                  }
                  if (isNil9 === false) {
                    var issuerInstance = issuerElement;
                    certificateInstance.issuer = issuerInstance;
                  }
                }
                
                var passwordElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (passwordElement) {
                  var isNil10 = false;
                  var nilAttribute10 = js2xml.getAttribute(responseDoc, passwordElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute10) {
                    isNil10 = nilAttribute10 === 'true';
                  }
                  if (isNil10 === false) {
                    var passwordInstance = passwordElement;
                    certificateInstance.password = passwordInstance;
                  }
                }
                
                var pfxBlobElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PFXBLOB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (pfxBlobElement) {
                  var isNil11 = false;
                  var nilAttribute11 = js2xml.getAttribute(responseDoc, pfxBlobElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute11) {
                    isNil11 = nilAttribute11 === 'true';
                  }
                  if (isNil11 === false) {
                    var pfxBlobInstance = new Buffer(pfxBlobElement, 'base64').toString('utf8');
                    certificateInstance.pfxBlob = pfxBlobInstance;
                  }
                }
                
                var selfLinkElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (selfLinkElement) {
                  var isNil12 = false;
                  var nilAttribute12 = js2xml.getAttribute(responseDoc, selfLinkElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute12) {
                    isNil12 = nilAttribute12 === 'true';
                  }
                  if (isNil12 === false) {
                    var selfLinkInstance = selfLinkElement;
                    certificateInstance.selfLinkUri = selfLinkInstance;
                  }
                }
                
                var siteNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (siteNameElement) {
                  var isNil13 = false;
                  var nilAttribute13 = js2xml.getAttribute(responseDoc, siteNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute13) {
                    isNil13 = nilAttribute13 === 'true';
                  }
                  if (isNil13 === false) {
                    var siteNameInstance = siteNameElement;
                    certificateInstance.siteName = siteNameInstance;
                  }
                }
                
                var subjectNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SUBJECTNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (subjectNameElement) {
                  var isNil14 = false;
                  var nilAttribute14 = js2xml.getAttribute(responseDoc, subjectNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute14) {
                    isNil14 = nilAttribute14 === 'true';
                  }
                  if (isNil14 === false) {
                    var subjectNameInstance = subjectNameElement;
                    certificateInstance.subjectName = subjectNameInstance;
                  }
                }
                
                var thumbprintElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (thumbprintElement2) {
                  var isNil15 = false;
                  var nilAttribute15 = js2xml.getAttribute(responseDoc, thumbprintElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute15) {
                    isNil15 = nilAttribute15 === 'true';
                  }
                  if (isNil15 === false) {
                    var thumbprintInstance2 = thumbprintElement2;
                    certificateInstance.thumbprint = thumbprintInstance2;
                  }
                }
                
                var toDeleteElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'TODELETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (toDeleteElement && (toDeleteElement === null || toDeleteElement === undefined || toDeleteElement.length === 0) === false) {
                  var isNil16 = false;
                  var nilAttribute16 = js2xml.getAttribute(responseDoc, toDeleteElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute16) {
                    isNil16 = nilAttribute16 === 'true';
                  }
                  if (isNil16 === false) {
                    var toDeleteInstance = toDeleteElement === 'true';
                    certificateInstance.isToBeDeleted = toDeleteInstance;
                  }
                }
                
                var validElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'VALID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (validElement && (validElement === null || validElement === undefined || validElement.length === 0) === false) {
                  var isNil17 = false;
                  var nilAttribute17 = js2xml.getAttribute(responseDoc, validElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute17) {
                    isNil17 = nilAttribute17 === 'true';
                  }
                  if (isNil17 === false) {
                    var validInstance = validElement === 'true';
                    certificateInstance.isValid = validInstance;
                  }
                }
              }
            }
          }
          
          var selfLinkElement2 = js2xml.getElement(responseDoc, siteElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (selfLinkElement2) {
            var selfLinkInstance2 = selfLinkElement2;
            webSiteInstance.uri = selfLinkInstance2;
          }
          
          var serverFarmElement = js2xml.getElement(responseDoc, siteElement, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serverFarmElement) {
            var serverFarmInstance = serverFarmElement;
            webSiteInstance.serverFarm = serverFarmInstance;
          }
          
          var siteModeElement = js2xml.getElement(responseDoc, siteElement, 'SITEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (siteModeElement) {
            var siteModeInstance = siteModeElement;
            webSiteInstance.siteMode = siteModeInstance;
          }
          
          var sitePropertiesElement = js2xml.getElement(responseDoc, siteElement, 'SITEPROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sitePropertiesElement) {
            var sitePropertiesInstance = { appSettings: {}, metadata: {}, properties: {} };
            webSiteInstance.siteProperties = sitePropertiesInstance;
            
            var appSettingsSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'APPSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (appSettingsSequenceElement) {
              for (var loweredIndex6 = 0; loweredIndex6 < js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex6 = loweredIndex6 + 1) {
                var appSettingsElement = js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex6];
                var appSettingsKey = js2xml.getElement(responseDoc, appSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var appSettingsValue = js2xml.getElement(responseDoc, appSettingsElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.appSettings[appSettingsKey] = appSettingsValue;
              }
            }
            
            var metadataSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'METADATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (metadataSequenceElement) {
              for (var loweredIndex7 = 0; loweredIndex7 < js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex7 = loweredIndex7 + 1) {
                var metadataElement = js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex7];
                var metadataKey = js2xml.getElement(responseDoc, metadataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var metadataValue = js2xml.getElement(responseDoc, metadataElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.metadata[metadataKey] = metadataValue;
              }
            }
            
            var propertiesSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'PROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (propertiesSequenceElement) {
              for (var loweredIndex8 = 0; loweredIndex8 < js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex8 = loweredIndex8 + 1) {
                var propertiesElement = js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex8];
                var propertiesKey = js2xml.getElement(responseDoc, propertiesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var propertiesValue = js2xml.getElement(responseDoc, propertiesElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.properties[propertiesKey] = propertiesValue;
              }
            }
          }
          
          var stateElement = js2xml.getElement(responseDoc, siteElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            webSiteInstance.state = stateInstance;
          }
          
          var usageStateElement = js2xml.getElement(responseDoc, siteElement, 'USAGESTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (usageStateElement) {
            var usageStateInstance = usageStateElement;
            webSiteInstance.usageState = usageStateInstance;
          }
          
          var webSpaceElement = js2xml.getElement(responseDoc, siteElement, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (webSpaceElement) {
            var webSpaceInstance = webSpaceElement;
            webSiteInstance.webSpace = webSpaceInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve the config settings for a web site by issuing an HTTP GET
   * request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getConfiguration = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/config';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { appSettings: {}, connectionStrings: [], defaultDocuments: [], handlerMappings: [], metadata: {} };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var siteConfigElement = js2xml.getElement(responseDoc, responseDoc, 'SITECONFIG', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (siteConfigElement) {
          var appSettingsSequenceElement = js2xml.getElement(responseDoc, siteConfigElement, 'APPSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (appSettingsSequenceElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
              var appSettingsElement = js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
              var appSettingsKey = js2xml.getElement(responseDoc, appSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              var appSettingsValue = js2xml.getElement(responseDoc, appSettingsElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              result.appSettings[appSettingsKey] = appSettingsValue;
            }
          }
          
          var connectionStringsSequenceElement = js2xml.getElement(responseDoc, siteConfigElement, 'CONNECTIONSTRINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (connectionStringsSequenceElement) {
            for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, connectionStringsSequenceElement, 'CONNSTRINGINFO', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex2 = loweredIndex2 + 1) {
              var connectionStringsElement = js2xml.getElements(responseDoc, connectionStringsSequenceElement, 'CONNSTRINGINFO', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex2];
              var connStringInfoInstance = {};
              result.connectionStrings.push(connStringInfoInstance);
              
              var connectionStringElement = js2xml.getElement(responseDoc, connectionStringsElement, 'CONNECTIONSTRING', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (connectionStringElement) {
                var connectionStringInstance = connectionStringElement;
                connStringInfoInstance.connectionString = connectionStringInstance;
              }
              
              var nameElement = js2xml.getElement(responseDoc, connectionStringsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement) {
                var nameInstance = nameElement;
                connStringInfoInstance.name = nameInstance;
              }
              
              var typeElement = js2xml.getElement(responseDoc, connectionStringsElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (typeElement) {
                var typeInstance = typeElement;
                connStringInfoInstance.type = typeInstance;
              }
            }
          }
          
          var defaultDocumentsSequenceElement = js2xml.getElement(responseDoc, siteConfigElement, 'DEFAULTDOCUMENTS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (defaultDocumentsSequenceElement) {
            for (var loweredIndex3 = 0; loweredIndex3 < js2xml.getElements(responseDoc, defaultDocumentsSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex3 = loweredIndex3 + 1) {
              var defaultDocumentsElement = js2xml.getElements(responseDoc, defaultDocumentsSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex3];
              result.defaultDocuments.push(defaultDocumentsElement);
            }
          }
          
          var detailedErrorLoggingEnabledElement = js2xml.getElement(responseDoc, siteConfigElement, 'DETAILEDERRORLOGGINGENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (detailedErrorLoggingEnabledElement && (detailedErrorLoggingEnabledElement === null || detailedErrorLoggingEnabledElement === undefined || detailedErrorLoggingEnabledElement.length === 0) === false) {
            var detailedErrorLoggingEnabledInstance = detailedErrorLoggingEnabledElement === 'true';
            result.detailedErrorLoggingEnabled = detailedErrorLoggingEnabledInstance;
          }
          
          var documentRootElement = js2xml.getElement(responseDoc, siteConfigElement, 'DOCUMENTROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (documentRootElement) {
            var documentRootInstance = documentRootElement;
            result.documentRoot = documentRootInstance;
          }
          
          var handlerMappingsSequenceElement = js2xml.getElement(responseDoc, siteConfigElement, 'HANDLERMAPPINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (handlerMappingsSequenceElement) {
            for (var loweredIndex4 = 0; loweredIndex4 < js2xml.getElements(responseDoc, handlerMappingsSequenceElement, 'HANDLERMAPPING', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex4 = loweredIndex4 + 1) {
              var handlerMappingsElement = js2xml.getElements(responseDoc, handlerMappingsSequenceElement, 'HANDLERMAPPING', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex4];
              var handlerMappingInstance = {};
              result.handlerMappings.push(handlerMappingInstance);
              
              var argumentsElement = js2xml.getElement(responseDoc, handlerMappingsElement, 'ARGUMENTS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (argumentsElement) {
                var argumentsInstance = argumentsElement;
                handlerMappingInstance.arguments = argumentsInstance;
              }
              
              var extensionElement = js2xml.getElement(responseDoc, handlerMappingsElement, 'EXTENSION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (extensionElement) {
                var extensionInstance = extensionElement;
                handlerMappingInstance.extension = extensionInstance;
              }
              
              var scriptProcessorElement = js2xml.getElement(responseDoc, handlerMappingsElement, 'SCRIPTPROCESSOR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (scriptProcessorElement) {
                var scriptProcessorInstance = scriptProcessorElement;
                handlerMappingInstance.scriptProcessor = scriptProcessorInstance;
              }
            }
          }
          
          var httpLoggingEnabledElement = js2xml.getElement(responseDoc, siteConfigElement, 'HTTPLOGGINGENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (httpLoggingEnabledElement && (httpLoggingEnabledElement === null || httpLoggingEnabledElement === undefined || httpLoggingEnabledElement.length === 0) === false) {
            var httpLoggingEnabledInstance = httpLoggingEnabledElement === 'true';
            result.httpLoggingEnabled = httpLoggingEnabledInstance;
          }
          
          var logsDirectorySizeLimitElement = js2xml.getElement(responseDoc, siteConfigElement, 'LOGSDIRECTORYSIZELIMIT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (logsDirectorySizeLimitElement && (logsDirectorySizeLimitElement === null || logsDirectorySizeLimitElement === undefined || logsDirectorySizeLimitElement.length === 0) === false) {
            var logsDirectorySizeLimitInstance = parseInt(logsDirectorySizeLimitElement, 10);
            result.logsDirectorySizeLimit = logsDirectorySizeLimitInstance;
          }
          
          var managedPipelineModeElement = js2xml.getElement(responseDoc, siteConfigElement, 'MANAGEDPIPELINEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (managedPipelineModeElement && (managedPipelineModeElement === null || managedPipelineModeElement === undefined || managedPipelineModeElement.length === 0) === false) {
            var managedPipelineModeInstance = managedPipelineModeElement;
            result.managedPipelineMode = managedPipelineModeInstance;
          }
          
          var metadataSequenceElement = js2xml.getElement(responseDoc, siteConfigElement, 'METADATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (metadataSequenceElement) {
            for (var loweredIndex5 = 0; loweredIndex5 < js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex5 = loweredIndex5 + 1) {
              var metadataElement = js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex5];
              var metadataKey = js2xml.getElement(responseDoc, metadataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              var metadataValue = js2xml.getElement(responseDoc, metadataElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              result.metadata[metadataKey] = metadataValue;
            }
          }
          
          var netFrameworkVersionElement = js2xml.getElement(responseDoc, siteConfigElement, 'NETFRAMEWORKVERSION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (netFrameworkVersionElement) {
            var netFrameworkVersionInstance = netFrameworkVersionElement;
            result.netFrameworkVersion = netFrameworkVersionInstance;
          }
          
          var numberOfWorkersElement = js2xml.getElement(responseDoc, siteConfigElement, 'NUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (numberOfWorkersElement && (numberOfWorkersElement === null || numberOfWorkersElement === undefined || numberOfWorkersElement.length === 0) === false) {
            var numberOfWorkersInstance = parseInt(numberOfWorkersElement, 10);
            result.numberOfWorkers = numberOfWorkersInstance;
          }
          
          var phpVersionElement = js2xml.getElement(responseDoc, siteConfigElement, 'PHPVERSION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (phpVersionElement) {
            var phpVersionInstance = phpVersionElement;
            result.phpVersion = phpVersionInstance;
          }
          
          var publishingPasswordElement = js2xml.getElement(responseDoc, siteConfigElement, 'PUBLISHINGPASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (publishingPasswordElement) {
            var publishingPasswordInstance = publishingPasswordElement;
            result.publishingPassword = publishingPasswordInstance;
          }
          
          var publishingUsernameElement = js2xml.getElement(responseDoc, siteConfigElement, 'PUBLISHINGUSERNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (publishingUsernameElement) {
            var publishingUsernameInstance = publishingUsernameElement;
            result.publishingUserName = publishingUsernameInstance;
          }
          
          var remoteDebuggingEnabledElement = js2xml.getElement(responseDoc, siteConfigElement, 'REMOTEDEBUGGINGENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (remoteDebuggingEnabledElement && (remoteDebuggingEnabledElement === null || remoteDebuggingEnabledElement === undefined || remoteDebuggingEnabledElement.length === 0) === false) {
            var remoteDebuggingEnabledInstance = remoteDebuggingEnabledElement === 'true';
            result.remoteDebuggingEnabled = remoteDebuggingEnabledInstance;
          }
          
          var remoteDebuggingVersionElement = js2xml.getElement(responseDoc, siteConfigElement, 'REMOTEDEBUGGINGVERSION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (remoteDebuggingVersionElement && (remoteDebuggingVersionElement === null || remoteDebuggingVersionElement === undefined || remoteDebuggingVersionElement.length === 0) === false) {
            var isNil = false;
            var nilAttribute = js2xml.getAttribute(responseDoc, remoteDebuggingVersionElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute) {
              isNil = nilAttribute === 'true';
            }
            if (isNil === false) {
              var remoteDebuggingVersionInstance = remoteDebuggingVersionElement;
              result.remoteDebuggingVersion = remoteDebuggingVersionInstance;
            }
          }
          
          var requestTracingEnabledElement = js2xml.getElement(responseDoc, siteConfigElement, 'REQUESTTRACINGENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (requestTracingEnabledElement && (requestTracingEnabledElement === null || requestTracingEnabledElement === undefined || requestTracingEnabledElement.length === 0) === false) {
            var requestTracingEnabledInstance = requestTracingEnabledElement === 'true';
            result.requestTracingEnabled = requestTracingEnabledInstance;
          }
          
          var requestTracingExpirationTimeElement = js2xml.getElement(responseDoc, siteConfigElement, 'REQUESTTRACINGEXPIRATIONTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (requestTracingExpirationTimeElement && (requestTracingExpirationTimeElement === null || requestTracingExpirationTimeElement === undefined || requestTracingExpirationTimeElement.length === 0) === false) {
            var isNil2 = false;
            var nilAttribute2 = js2xml.getAttribute(responseDoc, requestTracingExpirationTimeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute2) {
              isNil2 = nilAttribute2 === 'true';
            }
            if (isNil2 === false) {
              var requestTracingExpirationTimeInstance = new Date(requestTracingExpirationTimeElement);
              result.requestTracingExpirationTime = requestTracingExpirationTimeInstance;
            }
          }
          
          var scmTypeElement = js2xml.getElement(responseDoc, siteConfigElement, 'SCMTYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (scmTypeElement) {
            var scmTypeInstance = scmTypeElement;
            result.scmType = scmTypeInstance;
          }
          
          var use32BitWorkerProcessElement = js2xml.getElement(responseDoc, siteConfigElement, 'USE32BITWORKERPROCESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (use32BitWorkerProcessElement && (use32BitWorkerProcessElement === null || use32BitWorkerProcessElement === undefined || use32BitWorkerProcessElement.length === 0) === false) {
            var use32BitWorkerProcessInstance = use32BitWorkerProcessElement === 'true';
            result.use32BitWorkerProcess = use32BitWorkerProcessInstance;
          }
          
          var webSocketsEnabledElement = js2xml.getElement(responseDoc, siteConfigElement, 'WEBSOCKETSENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (webSocketsEnabledElement && (webSocketsEnabledElement === null || webSocketsEnabledElement === undefined || webSocketsEnabledElement.length === 0) === false) {
            var webSocketsEnabledInstance = webSocketsEnabledElement === 'true';
            result.webSocketsEnabled = webSocketsEnabledInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve historical usage metrics for a site by issuing an HTTP
   * GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166964.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {WebSiteGetHistoricalUsageMetricsParameters} parameters Parameters
   * supplied to the Get Historical Usage Metrics Web Site operation.
   * 
   * @param {array} [parameters.metricNames] Specifies a comma-separated list
   * of the names of the metrics to return. If the names parameter is not
   * specified, then all available metrics are returned.
   * 
   * @param {date} [parameters.startTime] The starting time of the metrics to
   * return. If this parameter is not specified, the beginning of the current
   * hour is used.
   * 
   * @param {date} [parameters.endTime] The ending time of the metrics to
   * return. If this parameter is not specified, the current time is used.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getHistoricalUsageMetrics = function(webSpaceName, webSiteName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/metrics?';
    if (parameters.metricNames && parameters.metricNames.length > 0) {
      url2 = url2 + '&names=' + encodeURIComponent(parameters.metricNames.join(','));
    }
    if (parameters.startTime) {
      url2 = url2 + '&StartTime=' + encodeURIComponent(parameters.startTime.toISOString());
    }
    if (parameters.endTime) {
      url2 = url2 + '&EndTime=' + encodeURIComponent(parameters.endTime.toISOString());
    }
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { usageMetrics: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var metricResponsesElement = js2xml.getElement(responseDoc, responseDoc, 'METRICRESPONSES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (metricResponsesElement) {
          if (metricResponsesElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, metricResponsesElement, 'METRICRESPONSE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
              var usageMetricsElement = js2xml.getElements(responseDoc, metricResponsesElement, 'METRICRESPONSE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
              var metricResponseInstance = {};
              result.usageMetrics.push(metricResponseInstance);
              
              var codeElement = js2xml.getElement(responseDoc, usageMetricsElement, 'CODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (codeElement) {
                var codeInstance = codeElement;
                metricResponseInstance.code = codeInstance;
              }
              
              var dataElement = js2xml.getElement(responseDoc, usageMetricsElement, 'DATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (dataElement) {
                var dataInstance = { values: [] };
                metricResponseInstance.data = dataInstance;
                
                var displayNameElement = js2xml.getElement(responseDoc, dataElement, 'DISPLAYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (displayNameElement) {
                  var displayNameInstance = displayNameElement;
                  dataInstance.displayName = displayNameInstance;
                }
                
                var endTimeElement = js2xml.getElement(responseDoc, dataElement, 'ENDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (endTimeElement) {
                  var endTimeInstance = new Date(endTimeElement);
                  dataInstance.endTime = endTimeInstance;
                }
                
                var nameElement = js2xml.getElement(responseDoc, dataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (nameElement) {
                  var nameInstance = nameElement;
                  dataInstance.name = nameInstance;
                }
                
                var primaryAggregationTypeElement = js2xml.getElement(responseDoc, dataElement, 'PRIMARYAGGREGATIONTYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (primaryAggregationTypeElement) {
                  var primaryAggregationTypeInstance = primaryAggregationTypeElement;
                  dataInstance.primaryAggregationType = primaryAggregationTypeInstance;
                }
                
                var startTimeElement = js2xml.getElement(responseDoc, dataElement, 'STARTTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (startTimeElement) {
                  var startTimeInstance = new Date(startTimeElement);
                  dataInstance.startTime = startTimeInstance;
                }
                
                var timeGrainElement = js2xml.getElement(responseDoc, dataElement, 'TIMEGRAIN', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (timeGrainElement) {
                  var timeGrainInstance = timeGrainElement;
                  dataInstance.timeGrain = timeGrainInstance;
                }
                
                var unitElement = js2xml.getElement(responseDoc, dataElement, 'UNIT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (unitElement) {
                  var unitInstance = unitElement;
                  dataInstance.unit = unitInstance;
                }
                
                var valuesSequenceElement = js2xml.getElement(responseDoc, dataElement, 'VALUES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (valuesSequenceElement) {
                  for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, valuesSequenceElement, 'METRICSAMPLE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex2 = loweredIndex2 + 1) {
                    var valuesElement = js2xml.getElements(responseDoc, valuesSequenceElement, 'METRICSAMPLE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex2];
                    var metricSampleInstance = {};
                    dataInstance.values.push(metricSampleInstance);
                    
                    var countElement = js2xml.getElement(responseDoc, valuesElement, 'COUNT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                    if (countElement) {
                      var countInstance = parseInt(countElement, 10);
                      metricSampleInstance.count = countInstance;
                    }
                    
                    var maximumElement = js2xml.getElement(responseDoc, valuesElement, 'MAXIMUM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                    if (maximumElement) {
                      var isNil = false;
                      var nilAttribute = js2xml.getAttribute(responseDoc, maximumElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                      if (nilAttribute) {
                        isNil = nilAttribute === 'true';
                      }
                      if (isNil === false) {
                        var maximumInstance = maximumElement;
                        metricSampleInstance.maximum = maximumInstance;
                      }
                    }
                    
                    var minimumElement = js2xml.getElement(responseDoc, valuesElement, 'MINIMUM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                    if (minimumElement) {
                      var isNil2 = false;
                      var nilAttribute2 = js2xml.getAttribute(responseDoc, minimumElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                      if (nilAttribute2) {
                        isNil2 = nilAttribute2 === 'true';
                      }
                      if (isNil2 === false) {
                        var minimumInstance = minimumElement;
                        metricSampleInstance.minimum = minimumInstance;
                      }
                    }
                    
                    var timeCreatedElement = js2xml.getElement(responseDoc, valuesElement, 'TIMECREATED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                    if (timeCreatedElement) {
                      var timeCreatedInstance = new Date(timeCreatedElement);
                      metricSampleInstance.timeCreated = timeCreatedInstance;
                    }
                    
                    var totalElement = js2xml.getElement(responseDoc, valuesElement, 'TOTAL', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                    if (totalElement) {
                      var totalInstance = totalElement;
                      metricSampleInstance.total = totalInstance;
                    }
                  }
                }
              }
              
              var messageElement = js2xml.getElement(responseDoc, usageMetricsElement, 'MESSAGE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (messageElement) {
                var messageInstance = messageElement;
                metricResponseInstance.message = messageInstance;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve the list of active instances by ids for a web site by
   * issuing an HTTP GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getInstanceIds = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/instanceids';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { instanceIds: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var arrayOfstringSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'ARRAYOFSTRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS');
        if (arrayOfstringSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, arrayOfstringSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex1 = loweredIndex1 + 1) {
            var arrayOfstringElement = js2xml.getElements(responseDoc, arrayOfstringSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex1];
            result.instanceIds.push(arrayOfstringElement);
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve the publish settings information for a web site by
   * issuing an HTTP GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166996.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getPublishProfile = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/publishxml';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { publishProfiles: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var publishDataElement = js2xml.getElement(responseDoc, responseDoc, 'PUBLISHDATA', '');
        if (publishDataElement) {
          if (publishDataElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, publishDataElement, 'PUBLISHPROFILE', '').length; loweredIndex1 = loweredIndex1 + 1) {
              var publishProfilesElement = js2xml.getElements(responseDoc, publishDataElement, 'PUBLISHPROFILE', '')[loweredIndex1];
              var publishProfileInstance = { databases: [] };
              result.publishProfiles.push(publishProfileInstance);
              
              var profileNameAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'PROFILENAME', '');
              if (profileNameAttribute) {
                publishProfileInstance.profileName = profileNameAttribute;
              }
              
              var publishMethodAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'PUBLISHMETHOD', '');
              if (publishMethodAttribute) {
                publishProfileInstance.publishMethod = publishMethodAttribute;
              }
              
              var publishUrlAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'PUBLISHURL', '');
              if (publishUrlAttribute) {
                publishProfileInstance.publishUrl = publishUrlAttribute;
              }
              
              var msdeploySiteAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'MSDEPLOYSITE', '');
              if (msdeploySiteAttribute) {
                publishProfileInstance.mSDeploySite = msdeploySiteAttribute;
              }
              
              var ftpPassiveModeAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'FTPPASSIVEMODE', '');
              if (ftpPassiveModeAttribute) {
                publishProfileInstance.ftpPassiveMode = ftpPassiveModeAttribute === 'true';
              }
              
              var userNameAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'USERNAME', '');
              if (userNameAttribute) {
                publishProfileInstance.userName = userNameAttribute;
              }
              
              var userPWDAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'USERPWD', '');
              if (userPWDAttribute) {
                publishProfileInstance.userPassword = userPWDAttribute;
              }
              
              var destinationAppUrlAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'DESTINATIONAPPURL', '');
              if (destinationAppUrlAttribute) {
                publishProfileInstance.destinationAppUri = destinationAppUrlAttribute;
              }
              
              var sQLServerDBConnectionStringAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'SQLSERVERDBCONNECTIONSTRING', '');
              if (sQLServerDBConnectionStringAttribute) {
                publishProfileInstance.sqlServerConnectionString = sQLServerDBConnectionStringAttribute;
              }
              
              var mySQLDBConnectionStringAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'MYSQLDBCONNECTIONSTRING', '');
              if (mySQLDBConnectionStringAttribute) {
                publishProfileInstance.mySqlConnectionString = mySQLDBConnectionStringAttribute;
              }
              
              var hostingProviderForumLinkAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'HOSTINGPROVIDERFORUMLINK', '');
              if (hostingProviderForumLinkAttribute) {
                publishProfileInstance.hostingProviderForumUri = hostingProviderForumLinkAttribute;
              }
              
              var controlPanelLinkAttribute = js2xml.getAttribute(responseDoc, publishProfilesElement, 'CONTROLPANELLINK', '');
              if (controlPanelLinkAttribute) {
                publishProfileInstance.controlPanelUri = controlPanelLinkAttribute;
              }
              
              var databasesSequenceElement = js2xml.getElement(responseDoc, publishProfilesElement, 'DATABASES', '');
              if (databasesSequenceElement) {
                for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, databasesSequenceElement, 'ADD', '').length; loweredIndex2 = loweredIndex2 + 1) {
                  var databasesElement = js2xml.getElements(responseDoc, databasesSequenceElement, 'ADD', '')[loweredIndex2];
                  var addInstance = {};
                  publishProfileInstance.databases.push(addInstance);
                  
                  var nameAttribute = js2xml.getAttribute(responseDoc, databasesElement, 'NAME', '');
                  if (nameAttribute) {
                    addInstance.name = nameAttribute;
                  }
                  
                  var connectionStringAttribute = js2xml.getAttribute(responseDoc, databasesElement, 'CONNECTIONSTRING', '');
                  if (connectionStringAttribute) {
                    addInstance.connectionString = connectionStringAttribute;
                  }
                  
                  var providerNameAttribute = js2xml.getAttribute(responseDoc, databasesElement, 'PROVIDERNAME', '');
                  if (providerNameAttribute) {
                    addInstance.providerName = providerNameAttribute;
                  }
                  
                  var typeAttribute = js2xml.getAttribute(responseDoc, databasesElement, 'TYPE', '');
                  if (typeAttribute) {
                    addInstance.type = typeAttribute;
                  }
                }
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * A web site repository is essentially a Git repository that you can use to
   * manage your web site content. By using Git source control tools, you can
   * push or pull version-controlled changes to your site. This API executes a
   * repository get operation.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getRepository = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/repository';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var anyURIElement = js2xml.getElement(responseDoc, responseDoc, 'ANYURI', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (anyURIElement) {
          result.uri = anyURIElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve current usage metrics for a site by issuing an HTTP GET
   * request. The metrics returned include CPU Time, Data In, Data Out, Local
   * Bytes Read, Local Bytes Written, Network Bytes Read, Network Bytes
   * Written, WP Stop Requests, Memory Usage, CPU Time - Minute Limit, and
   * File System Storage.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166991.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.getUsageMetrics = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/usages';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { usageMetrics: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var usagesElement = js2xml.getElement(responseDoc, responseDoc, 'USAGES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (usagesElement) {
          if (usagesElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, usagesElement, 'USAGE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
              var usageMetricsElement = js2xml.getElements(responseDoc, usagesElement, 'USAGE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
              var usageInstance = {};
              result.usageMetrics.push(usageInstance);
              
              var computeModeElement = js2xml.getElement(responseDoc, usageMetricsElement, 'COMPUTEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (computeModeElement) {
                var computeModeInstance = computeModeElement;
                usageInstance.computeMode = computeModeInstance;
              }
              
              var currentValueElement = js2xml.getElement(responseDoc, usageMetricsElement, 'CURRENTVALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (currentValueElement) {
                var currentValueInstance = currentValueElement;
                usageInstance.currentValue = currentValueInstance;
              }
              
              var displayNameElement = js2xml.getElement(responseDoc, usageMetricsElement, 'DISPLAYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (displayNameElement) {
                var displayNameInstance = displayNameElement;
                usageInstance.displayName = displayNameInstance;
              }
              
              var limitElement = js2xml.getElement(responseDoc, usageMetricsElement, 'LIMIT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (limitElement) {
                var limitInstance = limitElement;
                usageInstance.limit = limitInstance;
              }
              
              var nameElement = js2xml.getElement(responseDoc, usageMetricsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement) {
                var nameInstance = nameElement;
                usageInstance.name = nameInstance;
              }
              
              var nextResetTimeElement = js2xml.getElement(responseDoc, usageMetricsElement, 'NEXTRESETTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nextResetTimeElement) {
                var nextResetTimeInstance = new Date(nextResetTimeElement);
                usageInstance.nextResetTime = nextResetTimeInstance;
              }
              
              var resourceNameElement = js2xml.getElement(responseDoc, usageMetricsElement, 'RESOURCENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (resourceNameElement) {
                var resourceNameInstance = resourceNameElement;
                usageInstance.resourceName = resourceNameInstance;
              }
              
              var siteModeElement = js2xml.getElement(responseDoc, usageMetricsElement, 'SITEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (siteModeElement) {
                var siteModeInstance = siteModeElement;
                usageInstance.siteMode = siteModeInstance;
              }
              
              var unitElement = js2xml.getElement(responseDoc, usageMetricsElement, 'UNIT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (unitElement) {
                var unitInstance = unitElement;
                usageInstance.unit = unitInstance;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Determines if a host name is available.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.isHostnameAvailable = function(webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces?';
    url2 = url2 + 'ishostnameavailable=' + encodeURIComponent(webSiteName.trim());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var booleanElement = js2xml.getElement(responseDoc, responseDoc, 'BOOLEAN', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (booleanElement) {
          var booleanInstance = booleanElement === 'true';
          result.isAvailable = booleanInstance;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can restart a web site by issuing an HTTP POST request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn236425.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.restart = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/restart';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Length'] = '0';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * You can swap a web site from one slot to another slot.
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {string} sourceSlotName The name of the first web site slot to swap
   * (source).
   * 
   * @param {string} targetSlotName The name of the second web site slot to
   * swap with (target).
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.swapSlots = function(webSpaceName, webSiteName, sourceSlotName, targetSlotName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.webSites.beginSwapingSlots(webSpaceName, webSiteName, sourceSlotName, targetSlotName, function (err, result) {
      if (err) {
        return callback(err);
      }
      if (result.status === 'Succeeded') {
        return callback(null, result);
      }
      var delayInSeconds = 30;
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getOperationStatus(webSpaceName, webSiteName, result.operationId, function (err2, result2) {
            if (err2) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                if (result2.status !== 'Succeeded') {
                  if (result2.errors && result2.errors.length > 0) {
                    var ex = new Error(result2.errors[0].code + ' : ' + result2.errors[0].message);
                    ex.errorCode = result2.errors[0].code;
                    ex.errorMessage = result2.errors[0].message;
                    return callback(ex);
                  } else {
                    var ex = new Error('');
                    return callback(ex);
                  }
                }
                
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * A web site repository is essentially a Git repository that you can use to
   * manage your web site content. By using Git source control tools, you can
   * push or pull version-controlled changes to your site. This API executes a
   * repository sync operation.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.syncRepository = function(webSpaceName, webSiteName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/repository?action=sync';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Length'] = '0';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * You can update the settings for a web site by using the HTTP PUT method
   * and by specifying the settings in the request body.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn167005.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {WebSiteUpdateParameters} parameters Parameters supplied to the
   * Update Web Site operation.
   * 
   * @param {WebSpaceAvailabilityState} [parameters.availabilityState] The
   * state of the availability of management information for the web site.
   * Possible values are Normal or Limited. Normal means that the web site is
   * running correctly and that management information for the web site is
   * available. Limited means that only partial management information for the
   * web site is available and that detailed web site information is
   * unavailable.
   * 
   * @param {WebSiteComputeMode} [parameters.computeMode] The Compute Mode for
   * the web site. Possible values are Shared or Dedicated.
   * 
   * @param {boolean} [parameters.enabled] Indicates if the site is enabled.
   * Setting this value to false disables the site (takes the site offline).
   * 
   * @param {array} [parameters.hostNameSslStates] SSL states bound to the web
   * site.
   * 
   * @param {array} [parameters.hostNames] An array of strings that contains
   * the public host names for the web site, including custom domains.
   * Important: When you add a custom domain in a PUT operation, be sure to
   * include every host name that you want for the web site. To delete a
   * custom domain name in a PUT operation, include all of the host names for
   * the web site that you want to keep, but leave out the one that you want
   * to delete.
   * 
   * @param {WebSiteRuntimeAvailabilityState}
   * [parameters.runtimeAvailabilityState] The current runtime availability
   * state. Possible values are Normal, Degraded, or NotAvailable: Normal: the
   * web site is running correctly; Degraded: the web site is running
   * temporarily in a degraded mode (typically with less memory and a shared
   * instance). Not Available: due to an unexpected issue, the web site has
   * been excluded from provisioning. This typically occurs only for free
   * sites.
   * 
   * @param {array} [parameters.sslCertificates] SSL certificates bound to the
   * web site.
   * 
   * @param {string} [parameters.serverFarm] String specifying the server farm.
   * If a server farm exists, this value is DefaultServerFarm.
   * 
   * @param {WebSiteMode} [parameters.siteMode] String that represents the web
   * site mode. If the web site mode is Free, this value is Limited. If the
   * web site mode is Shared, this value is Basic. Note: The SiteMode value is
   * not used for Reserved mode. Reserved mode uses the ComputeMode setting.
   * 
   * @param {string} [parameters.state] A string that describes the state of
   * the web site. Possible values include Stopped, Running, or QuotaExceeded.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.update = function(webSpaceName, webSiteName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var siteElement = js2xml.createElement('Site', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, siteElement);
    
    if (parameters.availabilityState) {
      var availabilityStateElement = js2xml.createElement('AvailabilityState', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(availabilityStateElement, parameters.availabilityState ? parameters.availabilityState.toString() : 'Normal');
      js2xml.addChildElement(siteElement, availabilityStateElement);
    }
    
    if (parameters.computeMode) {
      var computeModeElement = js2xml.createElement('ComputeMode', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(computeModeElement, parameters.computeMode ? parameters.computeMode.toString() : 'Shared');
      js2xml.addChildElement(siteElement, computeModeElement);
    }
    
    if (parameters.enabled) {
      var enabledElement = js2xml.createElement('Enabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(enabledElement, parameters.enabled.toString());
      js2xml.addChildElement(siteElement, enabledElement);
    }
    
    if (parameters.hostNameSslStates) {
      var hostNameSslStatesSequenceElement = js2xml.createElement('HostNameSslStates', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex1 = 0; loweredIndex1 < parameters.hostNameSslStates.length; loweredIndex1 = loweredIndex1 + 1) {
        var hostNameSslStatesItem = parameters.hostNameSslStates[loweredIndex1];
        var webSiteHostNameSslStateElement = js2xml.createElement('WebSiteHostNameSslState', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(hostNameSslStatesSequenceElement, webSiteHostNameSslStateElement);
        
        if (hostNameSslStatesItem.sslState) {
          var sslStateElement = js2xml.createElement('SslState', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(sslStateElement, hostNameSslStatesItem.sslState ? hostNameSslStatesItem.sslState.toString() : 'Disabled');
          js2xml.addChildElement(webSiteHostNameSslStateElement, sslStateElement);
        }
        
        if (hostNameSslStatesItem.thumbprint) {
          var thumbprintElement = js2xml.createElement('Thumbprint', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(thumbprintElement, hostNameSslStatesItem.thumbprint);
          js2xml.addChildElement(webSiteHostNameSslStateElement, thumbprintElement);
        } else {
          var emptyElement = js2xml.createElement('Thumbprint', 'http://schemas.microsoft.com/windowsazure');
          var nilAttribute = js2xml.createAttribute('nil', 'http://www.w3.org/2001/XMLSchema-instance');
          js2xml.setAttributeValue(nilAttribute, 'true');
          js2xml.addAttribute(emptyElement, nilAttribute);
          js2xml.addChildElement(webSiteHostNameSslStateElement, emptyElement);
        }
        
        if (hostNameSslStatesItem.toUpdate) {
          var toUpdateElement = js2xml.createElement('ToUpdate', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(toUpdateElement, hostNameSslStatesItem.toUpdate.toString());
          js2xml.addChildElement(webSiteHostNameSslStateElement, toUpdateElement);
        }
      }
      js2xml.addChildElement(siteElement, hostNameSslStatesSequenceElement);
    }
    
    if (parameters.hostNames) {
      var hostNamesSequenceElement = js2xml.createElement('HostNames', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex2 = 0; loweredIndex2 < parameters.hostNames.length; loweredIndex2 = loweredIndex2 + 1) {
        var hostNamesItem = parameters.hostNames[loweredIndex2];
        var hostNamesItemElement = js2xml.createElement('string', 'http://schemas.microsoft.com/2003/10/Serialization/Arrays');
        js2xml.setElementValue(hostNamesItemElement, hostNamesItem);
        js2xml.addChildElement(hostNamesSequenceElement, hostNamesItemElement);
      }
      js2xml.addChildElement(siteElement, hostNamesSequenceElement);
    }
    
    if (parameters.runtimeAvailabilityState) {
      var runtimeAvailabilityStateElement = js2xml.createElement('RuntimeAvailabilityState', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(runtimeAvailabilityStateElement, parameters.runtimeAvailabilityState ? parameters.runtimeAvailabilityState.toString() : 'Normal');
      js2xml.addChildElement(siteElement, runtimeAvailabilityStateElement);
    }
    
    if (parameters.sslCertificates) {
      var sSLCertificatesSequenceElement = js2xml.createElement('SSLCertificates', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex3 = 0; loweredIndex3 < parameters.sslCertificates.length; loweredIndex3 = loweredIndex3 + 1) {
        var sSLCertificatesItem = parameters.sslCertificates[loweredIndex3];
        var certificateElement = js2xml.createElement('Certificate', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(sSLCertificatesSequenceElement, certificateElement);
        
        if (sSLCertificatesItem.password) {
          var passwordElement = js2xml.createElement('Password', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(passwordElement, sSLCertificatesItem.password);
          js2xml.addChildElement(certificateElement, passwordElement);
        }
        
        if (sSLCertificatesItem.pfxBlob) {
          var pfxBlobElement = js2xml.createElement('PfxBlob', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(pfxBlobElement, sSLCertificatesItem.pfxBlob.toString('base64'));
          js2xml.addChildElement(certificateElement, pfxBlobElement);
        }
        
        if (sSLCertificatesItem.thumbprint) {
          var thumbprintElement2 = js2xml.createElement('Thumbprint', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(thumbprintElement2, sSLCertificatesItem.thumbprint);
          js2xml.addChildElement(certificateElement, thumbprintElement2);
        }
        
        if (sSLCertificatesItem.isToBeDeleted) {
          var toDeleteElement = js2xml.createElement('ToDelete', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(toDeleteElement, sSLCertificatesItem.isToBeDeleted.toString());
          js2xml.addChildElement(certificateElement, toDeleteElement);
        }
      }
      js2xml.addChildElement(siteElement, sSLCertificatesSequenceElement);
    }
    
    if (parameters.serverFarm) {
      var serverFarmElement = js2xml.createElement('ServerFarm', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serverFarmElement, parameters.serverFarm);
      js2xml.addChildElement(siteElement, serverFarmElement);
    }
    
    if (parameters.siteMode) {
      var siteModeElement = js2xml.createElement('SiteMode', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(siteModeElement, parameters.siteMode ? parameters.siteMode.toString() : 'Limited');
      js2xml.addChildElement(siteElement, siteModeElement);
    }
    
    if (parameters.state) {
      var stateElement = js2xml.createElement('State', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(stateElement, parameters.state);
      js2xml.addChildElement(siteElement, stateElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var siteElement2 = js2xml.getElement(responseDoc, responseDoc, 'SITE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (siteElement2) {
          var webSiteInstance = { enabledHostNames: [], hostNames: [], hostNameSslStates: [], sslCertificates: [] };
          result.webSite = webSiteInstance;
          
          var adminEnabledElement = js2xml.getElement(responseDoc, siteElement2, 'ADMINENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (adminEnabledElement) {
            var adminEnabledInstance = adminEnabledElement === 'true';
            webSiteInstance.adminEnabled = adminEnabledInstance;
          }
          
          var availabilityStateElement2 = js2xml.getElement(responseDoc, siteElement2, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (availabilityStateElement2) {
            var availabilityStateInstance = availabilityStateElement2;
            webSiteInstance.availabilityState = availabilityStateInstance;
          }
          
          var computeModeElement2 = js2xml.getElement(responseDoc, siteElement2, 'COMPUTEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (computeModeElement2) {
            var computeModeInstance = computeModeElement2;
            webSiteInstance.computeMode = computeModeInstance;
          }
          
          var enabledElement2 = js2xml.getElement(responseDoc, siteElement2, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledElement2) {
            var enabledInstance = enabledElement2 === 'true';
            webSiteInstance.enabled = enabledInstance;
          }
          
          var enabledHostNamesSequenceElement = js2xml.getElement(responseDoc, siteElement2, 'ENABLEDHOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledHostNamesSequenceElement) {
            for (var loweredIndex4 = 0; loweredIndex4 < js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex4 = loweredIndex4 + 1) {
              var enabledHostNamesElement = js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex4];
              webSiteInstance.enabledHostNames.push(enabledHostNamesElement);
            }
          }
          
          var hostNameSslStatesSequenceElement2 = js2xml.getElement(responseDoc, siteElement2, 'HOSTNAMESSLSTATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNameSslStatesSequenceElement2) {
            for (var loweredIndex5 = 0; loweredIndex5 < js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement2, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex5 = loweredIndex5 + 1) {
              var hostNameSslStatesElement = js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement2, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex5];
              var webSiteHostNameSslStateInstance = {};
              webSiteInstance.hostNameSslStates.push(webSiteHostNameSslStateInstance);
              
              var nameElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement) {
                var nameInstance = nameElement;
                webSiteHostNameSslStateInstance.name = nameInstance;
              }
              
              var sslStateElement2 = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'SSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (sslStateElement2) {
                var sslStateInstance = sslStateElement2;
                webSiteHostNameSslStateInstance.sslState = sslStateInstance;
              }
              
              var thumbprintElement3 = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (thumbprintElement3) {
                var isNil = false;
                var nilAttribute2 = js2xml.getAttribute(responseDoc, thumbprintElement3, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute2) {
                  isNil = nilAttribute2 === 'true';
                }
                if (isNil === false) {
                  var thumbprintInstance = thumbprintElement3;
                  webSiteHostNameSslStateInstance.thumbprint = thumbprintInstance;
                }
              }
              
              var virtualIPElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'VIRTUALIP', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (virtualIPElement) {
                var isNil2 = false;
                var nilAttribute3 = js2xml.getAttribute(responseDoc, virtualIPElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute3) {
                  isNil2 = nilAttribute3 === 'true';
                }
                if (isNil2 === false) {
                  var virtualIPInstance = virtualIPElement;
                  webSiteHostNameSslStateInstance.virtualIP = virtualIPInstance;
                }
              }
            }
          }
          
          var hostNamesSequenceElement2 = js2xml.getElement(responseDoc, siteElement2, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (hostNamesSequenceElement2) {
            for (var loweredIndex6 = 0; loweredIndex6 < js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex6 = loweredIndex6 + 1) {
              var hostNamesElement = js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex6];
              webSiteInstance.hostNames.push(hostNamesElement);
            }
          }
          
          var lastModifiedTimeUtcElement = js2xml.getElement(responseDoc, siteElement2, 'LASTMODIFIEDTIMEUTC', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (lastModifiedTimeUtcElement) {
            var lastModifiedTimeUtcInstance = new Date(lastModifiedTimeUtcElement);
            webSiteInstance.lastModifiedTimeUtc = lastModifiedTimeUtcInstance;
          }
          
          var nameElement2 = js2xml.getElement(responseDoc, siteElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance2 = nameElement2;
            webSiteInstance.name = nameInstance2;
          }
          
          var ownerElement = js2xml.getElement(responseDoc, siteElement2, 'OWNER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (ownerElement) {
            var isNil3 = false;
            var nilAttribute4 = js2xml.getAttribute(responseDoc, ownerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute4) {
              isNil3 = nilAttribute4 === 'true';
            }
            if (isNil3 === false) {
              var ownerInstance = ownerElement;
              webSiteInstance.owner = ownerInstance;
            }
          }
          
          var repositorySiteNameElement = js2xml.getElement(responseDoc, siteElement2, 'REPOSITORYSITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (repositorySiteNameElement) {
            var repositorySiteNameInstance = repositorySiteNameElement;
            webSiteInstance.repositorySiteName = repositorySiteNameInstance;
          }
          
          var runtimeAvailabilityStateElement2 = js2xml.getElement(responseDoc, siteElement2, 'RUNTIMEAVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (runtimeAvailabilityStateElement2) {
            var runtimeAvailabilityStateInstance = runtimeAvailabilityStateElement2;
            webSiteInstance.runtimeAvailabilityState = runtimeAvailabilityStateInstance;
          }
          
          var sSLCertificatesSequenceElement2 = js2xml.getElement(responseDoc, siteElement2, 'SSLCERTIFICATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sSLCertificatesSequenceElement2) {
            var isNil4 = false;
            var nilAttribute5 = js2xml.getAttribute(responseDoc, sSLCertificatesSequenceElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute5) {
              isNil4 = nilAttribute5 === 'true';
            }
            if (isNil4 === false) {
              for (var loweredIndex7 = 0; loweredIndex7 < js2xml.getElements(responseDoc, sSLCertificatesSequenceElement2, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex7 = loweredIndex7 + 1) {
                var sSLCertificatesElement = js2xml.getElements(responseDoc, sSLCertificatesSequenceElement2, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex7];
                var certificateInstance = { hostNames: [] };
                webSiteInstance.sslCertificates.push(certificateInstance);
                
                var expirationDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'EXPIRATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (expirationDateElement && (expirationDateElement === null || expirationDateElement === undefined || expirationDateElement.length === 0) === false) {
                  var isNil5 = false;
                  var nilAttribute6 = js2xml.getAttribute(responseDoc, expirationDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute6) {
                    isNil5 = nilAttribute6 === 'true';
                  }
                  if (isNil5 === false) {
                    var expirationDateInstance = new Date(expirationDateElement);
                    certificateInstance.expirationDate = expirationDateInstance;
                  }
                }
                
                var friendlyNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'FRIENDLYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (friendlyNameElement) {
                  var isNil6 = false;
                  var nilAttribute7 = js2xml.getAttribute(responseDoc, friendlyNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute7) {
                    isNil6 = nilAttribute7 === 'true';
                  }
                  if (isNil6 === false) {
                    var friendlyNameInstance = friendlyNameElement;
                    certificateInstance.friendlyName = friendlyNameInstance;
                  }
                }
                
                var hostNamesSequenceElement3 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (hostNamesSequenceElement3) {
                  var isNil7 = false;
                  var nilAttribute8 = js2xml.getAttribute(responseDoc, hostNamesSequenceElement3, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute8) {
                    isNil7 = nilAttribute8 === 'true';
                  }
                  if (isNil7 === false) {
                    for (var loweredIndex8 = 0; loweredIndex8 < js2xml.getElements(responseDoc, hostNamesSequenceElement3, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex8 = loweredIndex8 + 1) {
                      var hostNamesElement2 = js2xml.getElements(responseDoc, hostNamesSequenceElement3, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex8];
                      certificateInstance.hostNames.push(hostNamesElement2);
                    }
                  }
                }
                
                var issueDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUEDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issueDateElement && (issueDateElement === null || issueDateElement === undefined || issueDateElement.length === 0) === false) {
                  var isNil8 = false;
                  var nilAttribute9 = js2xml.getAttribute(responseDoc, issueDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute9) {
                    isNil8 = nilAttribute9 === 'true';
                  }
                  if (isNil8 === false) {
                    var issueDateInstance = new Date(issueDateElement);
                    certificateInstance.issueDate = issueDateInstance;
                  }
                }
                
                var issuerElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (issuerElement) {
                  var isNil9 = false;
                  var nilAttribute10 = js2xml.getAttribute(responseDoc, issuerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute10) {
                    isNil9 = nilAttribute10 === 'true';
                  }
                  if (isNil9 === false) {
                    var issuerInstance = issuerElement;
                    certificateInstance.issuer = issuerInstance;
                  }
                }
                
                var passwordElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (passwordElement2) {
                  var isNil10 = false;
                  var nilAttribute11 = js2xml.getAttribute(responseDoc, passwordElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute11) {
                    isNil10 = nilAttribute11 === 'true';
                  }
                  if (isNil10 === false) {
                    var passwordInstance = passwordElement2;
                    certificateInstance.password = passwordInstance;
                  }
                }
                
                var pfxBlobElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PFXBLOB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (pfxBlobElement2) {
                  var isNil11 = false;
                  var nilAttribute12 = js2xml.getAttribute(responseDoc, pfxBlobElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute12) {
                    isNil11 = nilAttribute12 === 'true';
                  }
                  if (isNil11 === false) {
                    var pfxBlobInstance = new Buffer(pfxBlobElement2, 'base64').toString('utf8');
                    certificateInstance.pfxBlob = pfxBlobInstance;
                  }
                }
                
                var selfLinkElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (selfLinkElement) {
                  var isNil12 = false;
                  var nilAttribute13 = js2xml.getAttribute(responseDoc, selfLinkElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute13) {
                    isNil12 = nilAttribute13 === 'true';
                  }
                  if (isNil12 === false) {
                    var selfLinkInstance = selfLinkElement;
                    certificateInstance.selfLinkUri = selfLinkInstance;
                  }
                }
                
                var siteNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (siteNameElement) {
                  var isNil13 = false;
                  var nilAttribute14 = js2xml.getAttribute(responseDoc, siteNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute14) {
                    isNil13 = nilAttribute14 === 'true';
                  }
                  if (isNil13 === false) {
                    var siteNameInstance = siteNameElement;
                    certificateInstance.siteName = siteNameInstance;
                  }
                }
                
                var subjectNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SUBJECTNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (subjectNameElement) {
                  var isNil14 = false;
                  var nilAttribute15 = js2xml.getAttribute(responseDoc, subjectNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute15) {
                    isNil14 = nilAttribute15 === 'true';
                  }
                  if (isNil14 === false) {
                    var subjectNameInstance = subjectNameElement;
                    certificateInstance.subjectName = subjectNameInstance;
                  }
                }
                
                var thumbprintElement4 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (thumbprintElement4) {
                  var isNil15 = false;
                  var nilAttribute16 = js2xml.getAttribute(responseDoc, thumbprintElement4, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute16) {
                    isNil15 = nilAttribute16 === 'true';
                  }
                  if (isNil15 === false) {
                    var thumbprintInstance2 = thumbprintElement4;
                    certificateInstance.thumbprint = thumbprintInstance2;
                  }
                }
                
                var toDeleteElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'TODELETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (toDeleteElement2 && (toDeleteElement2 === null || toDeleteElement2 === undefined || toDeleteElement2.length === 0) === false) {
                  var isNil16 = false;
                  var nilAttribute17 = js2xml.getAttribute(responseDoc, toDeleteElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute17) {
                    isNil16 = nilAttribute17 === 'true';
                  }
                  if (isNil16 === false) {
                    var toDeleteInstance = toDeleteElement2 === 'true';
                    certificateInstance.isToBeDeleted = toDeleteInstance;
                  }
                }
                
                var validElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'VALID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (validElement && (validElement === null || validElement === undefined || validElement.length === 0) === false) {
                  var isNil17 = false;
                  var nilAttribute18 = js2xml.getAttribute(responseDoc, validElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute18) {
                    isNil17 = nilAttribute18 === 'true';
                  }
                  if (isNil17 === false) {
                    var validInstance = validElement === 'true';
                    certificateInstance.isValid = validInstance;
                  }
                }
              }
            }
          }
          
          var selfLinkElement2 = js2xml.getElement(responseDoc, siteElement2, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (selfLinkElement2) {
            var selfLinkInstance2 = selfLinkElement2;
            webSiteInstance.uri = selfLinkInstance2;
          }
          
          var serverFarmElement2 = js2xml.getElement(responseDoc, siteElement2, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serverFarmElement2) {
            var serverFarmInstance = serverFarmElement2;
            webSiteInstance.serverFarm = serverFarmInstance;
          }
          
          var siteModeElement2 = js2xml.getElement(responseDoc, siteElement2, 'SITEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (siteModeElement2) {
            var siteModeInstance = siteModeElement2;
            webSiteInstance.siteMode = siteModeInstance;
          }
          
          var sitePropertiesElement = js2xml.getElement(responseDoc, siteElement2, 'SITEPROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sitePropertiesElement) {
            var sitePropertiesInstance = { appSettings: {}, metadata: {}, properties: {} };
            webSiteInstance.siteProperties = sitePropertiesInstance;
            
            var appSettingsSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'APPSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (appSettingsSequenceElement) {
              for (var loweredIndex9 = 0; loweredIndex9 < js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex9 = loweredIndex9 + 1) {
                var appSettingsElement = js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex9];
                var appSettingsKey = js2xml.getElement(responseDoc, appSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var appSettingsValue = js2xml.getElement(responseDoc, appSettingsElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.appSettings[appSettingsKey] = appSettingsValue;
              }
            }
            
            var metadataSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'METADATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (metadataSequenceElement) {
              for (var loweredIndex10 = 0; loweredIndex10 < js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex10 = loweredIndex10 + 1) {
                var metadataElement = js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex10];
                var metadataKey = js2xml.getElement(responseDoc, metadataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var metadataValue = js2xml.getElement(responseDoc, metadataElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.metadata[metadataKey] = metadataValue;
              }
            }
            
            var propertiesSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'PROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (propertiesSequenceElement) {
              for (var loweredIndex11 = 0; loweredIndex11 < js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex11 = loweredIndex11 + 1) {
                var propertiesElement = js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex11];
                var propertiesKey = js2xml.getElement(responseDoc, propertiesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                var propertiesValue = js2xml.getElement(responseDoc, propertiesElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                sitePropertiesInstance.properties[propertiesKey] = propertiesValue;
              }
            }
          }
          
          var stateElement2 = js2xml.getElement(responseDoc, siteElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement2) {
            var stateInstance = stateElement2;
            webSiteInstance.state = stateInstance;
          }
          
          var usageStateElement = js2xml.getElement(responseDoc, siteElement2, 'USAGESTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (usageStateElement) {
            var usageStateInstance = usageStateElement;
            webSiteInstance.usageState = usageStateInstance;
          }
          
          var webSpaceElement = js2xml.getElement(responseDoc, siteElement2, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (webSpaceElement) {
            var webSpaceInstance = webSpaceElement;
            webSiteInstance.webSpace = webSpaceInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can update the config settings for a web site by issuing an HTTP PUT
   * with a request body containing the updated settings.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {string} webSiteName The name of the web site.
   * 
   * @param {WebSiteUpdateConfigurationParameters} parameters Parameters
   * supplied to the Update Configuration Web Site operation.
   * 
   * @param {object} [parameters.appSettings] A set of name/value pairs that
   * contain application settings for a web site.
   * 
   * @param {array} [parameters.connectionStrings] Connection strings for
   * database and other external resources.
   * 
   * @param {array} [parameters.defaultDocuments] Elements that list, in order
   * of preference, the name of the file that a web site returns when the web
   * site's domain name is requested by itself. For example, if the default
   * document for http://contoso.com is default.htm, the page
   * http://www.contoso.com/default.htm is returned when the browser is
   * pointed to http://www.contoso.com.
   * 
   * @param {boolean} [parameters.detailedErrorLoggingEnabled] Indicated if
   * detailed error logging is enabled.
   * 
   * @param {string} [parameters.documentRoot] The document root.
   * 
   * @param {array} [parameters.handlerMappings] Specifies custom executable
   * programs for handling requests for specific file name extensions.
   * 
   * @param {boolean} [parameters.httpLoggingEnabled] Indicates if HTTP error
   * logging is enabled.
   * 
   * @param {number} [parameters.logsDirectorySizeLimit] The size limit of the
   * logs directory.
   * 
   * @param {ManagedPipelineMode} [parameters.managedPipelineMode] The managed
   * pipeline mode.
   * 
   * @param {object} [parameters.metadata] Name/value pairs for source control
   * or other information.
   * 
   * @param {string} [parameters.netFrameworkVersion] The .NET Framework
   * version. Supported values are v2.0 and v4.0.
   * 
   * @param {number} [parameters.numberOfWorkers] The number of web workers
   * allotted to the web site. If the site mode is Free, this value is 1. If
   * the site mode is Shared, this value can range from 1 through 6. If the
   * site mode is Standard, this value can range from 1 through 10.
   * 
   * @param {string} [parameters.phpVersion] The web site's PHP version.
   * Supported values are an empty string (an empty string disables PHP), 5.3,
   * and 5.4.
   * 
   * @param {string} [parameters.publishingPassword] Hash value of the password
   * used for publishing the web site.
   * 
   * @param {string} [parameters.publishingUserName] The user name used for
   * publishing the web site. This is normally a dollar sign prepended to the
   * web site name (for example, "$contoso").
   * 
   * @param {boolean} [parameters.remoteDebuggingEnabled] Indicates if remote
   * debugging is enabled.
   * 
   * @param {RemoteDebuggingVersion} parameters.remoteDebuggingVersion The
   * remote debugging version.
   * 
   * @param {boolean} [parameters.requestTracingEnabled] Indicates if request
   * tracing is enabled.
   * 
   * @param {date} [parameters.requestTracingExpirationTime] Time remaining
   * until request tracing expires.
   * 
   * @param {string} [parameters.scmType] The source control method that the
   * web site is using (for example, Local Git). If deployment from source
   * control has not been set up for the web site, this value is None.
   * 
   * @param {boolean} [parameters.use32BitWorkerProcess] Indicates if 32-bit
   * mode is enabled.
   * 
   * @param {boolean} [parameters.webSocketsEnabled] Indicates if Web Sockets
   * are enabled.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSiteOperations.prototype.updateConfiguration = function(webSpaceName, webSiteName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    if (webSiteName === null || webSiteName === undefined) {
      return callback(new Error('webSiteName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites/' + webSiteName.trim() + '/config';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var siteConfigElement = js2xml.createElement('SiteConfig', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, siteConfigElement);
    
    if (parameters.appSettings) {
      var appSettingsDictionaryElement = js2xml.createElement('AppSettings', 'http://schemas.microsoft.com/windowsazure');
      for (var appSettingsKey in parameters.appSettings) {
        var appSettingsValue = parameters.appSettings[appSettingsKey];
        var appSettingsElement = js2xml.createElement('NameValuePair', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(appSettingsDictionaryElement, appSettingsElement);
        
        var appSettingsKeyElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
        js2xml.setElementValue(appSettingsKeyElement, appSettingsKey);
        js2xml.addChildElement(appSettingsElement, appSettingsKeyElement);
        
        var appSettingsValueElement = js2xml.createElement('Value', 'http://schemas.microsoft.com/windowsazure');
        js2xml.setElementValue(appSettingsValueElement, appSettingsValue);
        js2xml.addChildElement(appSettingsElement, appSettingsValueElement);
      }
      js2xml.addChildElement(siteConfigElement, appSettingsDictionaryElement);
    }
    
    if (parameters.connectionStrings) {
      var connectionStringsSequenceElement = js2xml.createElement('ConnectionStrings', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex1 = 0; loweredIndex1 < parameters.connectionStrings.length; loweredIndex1 = loweredIndex1 + 1) {
        var connectionStringsItem = parameters.connectionStrings[loweredIndex1];
        var connStringInfoElement = js2xml.createElement('ConnStringInfo', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(connectionStringsSequenceElement, connStringInfoElement);
        
        if (connectionStringsItem.connectionString) {
          var connectionStringElement = js2xml.createElement('ConnectionString', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(connectionStringElement, connectionStringsItem.connectionString);
          js2xml.addChildElement(connStringInfoElement, connectionStringElement);
        }
        
        if (connectionStringsItem.name) {
          var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(nameElement, connectionStringsItem.name);
          js2xml.addChildElement(connStringInfoElement, nameElement);
        }
        
        if (connectionStringsItem.type) {
          var typeElement = js2xml.createElement('Type', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(typeElement, connectionStringsItem.type);
          js2xml.addChildElement(connStringInfoElement, typeElement);
        }
      }
      js2xml.addChildElement(siteConfigElement, connectionStringsSequenceElement);
    }
    
    if (parameters.defaultDocuments) {
      var defaultDocumentsSequenceElement = js2xml.createElement('DefaultDocuments', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex2 = 0; loweredIndex2 < parameters.defaultDocuments.length; loweredIndex2 = loweredIndex2 + 1) {
        var defaultDocumentsItem = parameters.defaultDocuments[loweredIndex2];
        var defaultDocumentsItemElement = js2xml.createElement('string', 'http://schemas.microsoft.com/2003/10/Serialization/Arrays');
        js2xml.setElementValue(defaultDocumentsItemElement, defaultDocumentsItem);
        js2xml.addChildElement(defaultDocumentsSequenceElement, defaultDocumentsItemElement);
      }
      js2xml.addChildElement(siteConfigElement, defaultDocumentsSequenceElement);
    }
    
    if (parameters.detailedErrorLoggingEnabled) {
      var detailedErrorLoggingEnabledElement = js2xml.createElement('DetailedErrorLoggingEnabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(detailedErrorLoggingEnabledElement, parameters.detailedErrorLoggingEnabled.toString());
      js2xml.addChildElement(siteConfigElement, detailedErrorLoggingEnabledElement);
    }
    
    if (parameters.documentRoot) {
      var documentRootElement = js2xml.createElement('DocumentRoot', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(documentRootElement, parameters.documentRoot);
      js2xml.addChildElement(siteConfigElement, documentRootElement);
    }
    
    if (parameters.handlerMappings) {
      var handlerMappingsSequenceElement = js2xml.createElement('HandlerMappings', 'http://schemas.microsoft.com/windowsazure');
      for (var loweredIndex3 = 0; loweredIndex3 < parameters.handlerMappings.length; loweredIndex3 = loweredIndex3 + 1) {
        var handlerMappingsItem = parameters.handlerMappings[loweredIndex3];
        var handlerMappingElement = js2xml.createElement('HandlerMapping', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(handlerMappingsSequenceElement, handlerMappingElement);
        
        if (handlerMappingsItem.arguments) {
          var argumentsElement = js2xml.createElement('Arguments', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(argumentsElement, handlerMappingsItem.arguments);
          js2xml.addChildElement(handlerMappingElement, argumentsElement);
        }
        
        if (handlerMappingsItem.extension) {
          var extensionElement = js2xml.createElement('Extension', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(extensionElement, handlerMappingsItem.extension);
          js2xml.addChildElement(handlerMappingElement, extensionElement);
        }
        
        if (handlerMappingsItem.scriptProcessor) {
          var scriptProcessorElement = js2xml.createElement('ScriptProcessor', 'http://schemas.microsoft.com/windowsazure');
          js2xml.setElementValue(scriptProcessorElement, handlerMappingsItem.scriptProcessor);
          js2xml.addChildElement(handlerMappingElement, scriptProcessorElement);
        }
      }
      js2xml.addChildElement(siteConfigElement, handlerMappingsSequenceElement);
    }
    
    if (parameters.httpLoggingEnabled) {
      var httpLoggingEnabledElement = js2xml.createElement('HttpLoggingEnabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(httpLoggingEnabledElement, parameters.httpLoggingEnabled.toString());
      js2xml.addChildElement(siteConfigElement, httpLoggingEnabledElement);
    }
    
    if (parameters.logsDirectorySizeLimit) {
      var logsDirectorySizeLimitElement = js2xml.createElement('LogsDirectorySizeLimit', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(logsDirectorySizeLimitElement, parameters.logsDirectorySizeLimit.toString());
      js2xml.addChildElement(siteConfigElement, logsDirectorySizeLimitElement);
    }
    
    if (parameters.managedPipelineMode) {
      var managedPipelineModeElement = js2xml.createElement('ManagedPipelineMode', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(managedPipelineModeElement, parameters.managedPipelineMode ? parameters.managedPipelineMode.toString() : 'Integrated');
      js2xml.addChildElement(siteConfigElement, managedPipelineModeElement);
    }
    
    if (parameters.metadata) {
      var metadataDictionaryElement = js2xml.createElement('Metadata', 'http://schemas.microsoft.com/windowsazure');
      for (var metadataKey in parameters.metadata) {
        var metadataValue = parameters.metadata[metadataKey];
        var metadataElement = js2xml.createElement('NameValuePair', 'http://schemas.microsoft.com/windowsazure');
        js2xml.addChildElement(metadataDictionaryElement, metadataElement);
        
        var metadataKeyElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
        js2xml.setElementValue(metadataKeyElement, metadataKey);
        js2xml.addChildElement(metadataElement, metadataKeyElement);
        
        var metadataValueElement = js2xml.createElement('Value', 'http://schemas.microsoft.com/windowsazure');
        js2xml.setElementValue(metadataValueElement, metadataValue);
        js2xml.addChildElement(metadataElement, metadataValueElement);
      }
      js2xml.addChildElement(siteConfigElement, metadataDictionaryElement);
    }
    
    if (parameters.netFrameworkVersion) {
      var netFrameworkVersionElement = js2xml.createElement('NetFrameworkVersion', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(netFrameworkVersionElement, parameters.netFrameworkVersion);
      js2xml.addChildElement(siteConfigElement, netFrameworkVersionElement);
    }
    
    if (parameters.numberOfWorkers) {
      var numberOfWorkersElement = js2xml.createElement('NumberOfWorkers', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(numberOfWorkersElement, parameters.numberOfWorkers.toString());
      js2xml.addChildElement(siteConfigElement, numberOfWorkersElement);
    }
    
    if (parameters.phpVersion) {
      var phpVersionElement = js2xml.createElement('PhpVersion', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(phpVersionElement, parameters.phpVersion);
      js2xml.addChildElement(siteConfigElement, phpVersionElement);
    }
    
    if (parameters.publishingPassword) {
      var publishingPasswordElement = js2xml.createElement('PublishingPassword', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(publishingPasswordElement, parameters.publishingPassword);
      js2xml.addChildElement(siteConfigElement, publishingPasswordElement);
    }
    
    if (parameters.publishingUserName) {
      var publishingUsernameElement = js2xml.createElement('PublishingUsername', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(publishingUsernameElement, parameters.publishingUserName);
      js2xml.addChildElement(siteConfigElement, publishingUsernameElement);
    }
    
    if (parameters.remoteDebuggingEnabled) {
      var remoteDebuggingEnabledElement = js2xml.createElement('RemoteDebuggingEnabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(remoteDebuggingEnabledElement, parameters.remoteDebuggingEnabled.toString());
      js2xml.addChildElement(siteConfigElement, remoteDebuggingEnabledElement);
    }
    
    var remoteDebuggingVersionElement = js2xml.createElement('RemoteDebuggingVersion', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(remoteDebuggingVersionElement, parameters.remoteDebuggingVersion ? parameters.remoteDebuggingVersion.toString() : 'VS2012');
    js2xml.addChildElement(siteConfigElement, remoteDebuggingVersionElement);
    
    if (parameters.requestTracingEnabled) {
      var requestTracingEnabledElement = js2xml.createElement('RequestTracingEnabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(requestTracingEnabledElement, parameters.requestTracingEnabled.toString());
      js2xml.addChildElement(siteConfigElement, requestTracingEnabledElement);
    }
    
    if (parameters.requestTracingExpirationTime) {
      var requestTracingExpirationTimeElement = js2xml.createElement('RequestTracingExpirationTime', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(requestTracingExpirationTimeElement, parameters.requestTracingExpirationTime.toISOString());
      js2xml.addChildElement(siteConfigElement, requestTracingExpirationTimeElement);
    }
    
    if (parameters.scmType) {
      var scmTypeElement = js2xml.createElement('ScmType', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(scmTypeElement, parameters.scmType);
      js2xml.addChildElement(siteConfigElement, scmTypeElement);
    }
    
    if (parameters.use32BitWorkerProcess) {
      var use32BitWorkerProcessElement = js2xml.createElement('Use32BitWorkerProcess', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(use32BitWorkerProcessElement, parameters.use32BitWorkerProcess.toString());
      js2xml.addChildElement(siteConfigElement, use32BitWorkerProcessElement);
    }
    
    if (parameters.webSocketsEnabled) {
      var webSocketsEnabledElement = js2xml.createElement('WebSocketsEnabled', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(webSocketsEnabledElement, parameters.webSocketsEnabled.toString());
      js2xml.addChildElement(siteConfigElement, webSocketsEnabledElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return WebSiteOperations;
})();

var WebSpaceOperations = ( /** @lends WebSpaceOperations */ function() {
  /**
   * @class
   * Operations for managing web spaces beneath your subscription.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [WebSiteManagementClient] {@link
   * WebSiteManagementClient~WebSiteManagementClient}.
   * See [webSpaces] {@link
   * WebSiteManagementClient~WebSiteManagementClient#webSpaces}.
   * Initializes a new instance of the WebSpaceOperations class.
   * @constructor
   * 
   * @param {WebSiteManagementClient} client Reference to the service client.
   */
  function WebSpaceOperations(client) {
    this.client = client;
  }
  
  /**
   * Creates a source control user with permissions to publish to this web
   * space.
   * 
   * @param {string} username The user name.
   * 
   * @param {string} password The user password.
   * 
   * @param {WebSpacesCreatePublishingUserParameters} [parametersOrCallback]
   * Parameters supplied to the Create Publishing User operation.
   * 
   * @param {string} [parametersOrCallback.name] The publishing user name.
   * 
   * @param {string} [parametersOrCallback.publishingPassword] The publishing
   * user password.
   * 
   * @param {string} [parametersOrCallback.publishingUserName] The publishing
   * user username.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.createPublishingUser = function(username, password, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (username === null || username === undefined) {
      return callback(new Error('username cannot be null.'));
    }
    if (password === null || password === undefined) {
      return callback(new Error('password cannot be null.'));
    }
    if (parameters) {
      if (parameters.publishingPassword === null || parameters.publishingPassword === undefined) {
        return callback(new Error('parameters.publishingPassword cannot be null.'));
      }
      if (parameters.publishingUserName === null || parameters.publishingUserName === undefined) {
        return callback(new Error('parameters.publishingUserName cannot be null.'));
      }
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces?properties=publishingCredentials';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    if (parameters) {
      var userElement = js2xml.createElement('User', 'http://schemas.microsoft.com/windowsazure');
      js2xml.addChildElement(requestDoc, userElement);
      
      if (parameters.name) {
        var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
        js2xml.setElementValue(nameElement, parameters.name);
        js2xml.addChildElement(userElement, nameElement);
      }
      
      var publishingPasswordElement = js2xml.createElement('PublishingPassword', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(publishingPasswordElement, parameters.publishingPassword);
      js2xml.addChildElement(userElement, publishingPasswordElement);
      
      var publishingUserNameElement = js2xml.createElement('PublishingUserName', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(publishingUserNameElement, parameters.publishingUserName);
      js2xml.addChildElement(userElement, publishingUserNameElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var userElement2 = js2xml.getElement(responseDoc, responseDoc, 'USER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (userElement2) {
          var nameElement2 = js2xml.getElement(responseDoc, userElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var publishingPasswordElement2 = js2xml.getElement(responseDoc, userElement2, 'PUBLISHINGPASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (publishingPasswordElement2) {
            var publishingPasswordInstance = publishingPasswordElement2;
            result.publishingPassword = publishingPasswordInstance;
          }
          
          var publishingUserNameElement2 = js2xml.getElement(responseDoc, userElement2, 'PUBLISHINGUSERNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (publishingUserNameElement2) {
            var publishingUserNameInstance = publishingUserNameElement2;
            result.publishingUserName = publishingUserNameInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve details for a specified web space name by issuing an HTTP
   * GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn167017.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.get = function(webSpaceName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var webSpacesElement = js2xml.getElement(responseDoc, responseDoc, 'WEBSPACES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (webSpacesElement) {
          var availabilityStateElement = js2xml.getElement(responseDoc, webSpacesElement, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (availabilityStateElement) {
            var availabilityStateInstance = availabilityStateElement;
            result.availabilityState = availabilityStateInstance;
          }
          
          var currentNumberOfWorkersElement = js2xml.getElement(responseDoc, webSpacesElement, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentNumberOfWorkersElement && (currentNumberOfWorkersElement === null || currentNumberOfWorkersElement === undefined || currentNumberOfWorkersElement.length === 0) === false) {
            var isNil = false;
            var nilAttribute = js2xml.getAttribute(responseDoc, currentNumberOfWorkersElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute) {
              isNil = nilAttribute === 'true';
            }
            if (isNil === false) {
              var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement, 10);
              result.currentNumberOfWorkers = currentNumberOfWorkersInstance;
            }
          }
          
          var currentWorkerSizeElement = js2xml.getElement(responseDoc, webSpacesElement, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (currentWorkerSizeElement && (currentWorkerSizeElement === null || currentWorkerSizeElement === undefined || currentWorkerSizeElement.length === 0) === false) {
            var isNil2 = false;
            var nilAttribute2 = js2xml.getAttribute(responseDoc, currentWorkerSizeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
            if (nilAttribute2) {
              isNil2 = nilAttribute2 === 'true';
            }
            if (isNil2 === false) {
              var currentWorkerSizeInstance = currentWorkerSizeElement;
              result.currentWorkerSize = currentWorkerSizeInstance;
            }
          }
          
          var geoLocationElement = js2xml.getElement(responseDoc, webSpacesElement, 'GEOLOCATION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (geoLocationElement) {
            var geoLocationInstance = geoLocationElement;
            result.geoLocation = geoLocationInstance;
          }
          
          var geoRegionElement = js2xml.getElement(responseDoc, webSpacesElement, 'GEOREGION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (geoRegionElement) {
            var geoRegionInstance = geoRegionElement;
            result.geoRegion = geoRegionInstance;
          }
          
          var nameElement = js2xml.getElement(responseDoc, webSpacesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            result.name = nameInstance;
          }
          
          var planElement = js2xml.getElement(responseDoc, webSpacesElement, 'PLAN', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (planElement) {
            var planInstance = planElement;
            result.plan = planInstance;
          }
          
          var statusElement = js2xml.getElement(responseDoc, webSpacesElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (statusElement) {
            var statusInstance = statusElement;
            result.status = statusInstance;
          }
          
          var subscriptionElement = js2xml.getElement(responseDoc, webSpacesElement, 'SUBSCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (subscriptionElement) {
            var subscriptionInstance = subscriptionElement;
            result.subscription = subscriptionInstance;
          }
          
          var workerSizeElement = js2xml.getElement(responseDoc, webSpacesElement, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (workerSizeElement && (workerSizeElement === null || workerSizeElement === undefined || workerSizeElement.length === 0) === false) {
            var workerSizeInstance = workerSizeElement;
            result.workerSize = workerSizeInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Get the DNS Suffix for this subscription.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.getDnsSuffix = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces?properties=dnssuffix';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var stringElement = js2xml.getElement(responseDoc, responseDoc, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (stringElement) {
          result.dnsSuffix = stringElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can list the web spaces under the current subscription by issuing a
   * GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn166961.aspx for
   * more information)
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.list = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { webSpaces: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var webSpacesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'WEBSPACES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (webSpacesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, webSpacesSequenceElement, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var webSpacesElement = js2xml.getElements(responseDoc, webSpacesSequenceElement, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var webSpaceInstance = {};
            result.webSpaces.push(webSpaceInstance);
            
            var availabilityStateElement = js2xml.getElement(responseDoc, webSpacesElement, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (availabilityStateElement) {
              var availabilityStateInstance = availabilityStateElement;
              webSpaceInstance.availabilityState = availabilityStateInstance;
            }
            
            var currentNumberOfWorkersElement = js2xml.getElement(responseDoc, webSpacesElement, 'CURRENTNUMBEROFWORKERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (currentNumberOfWorkersElement && (currentNumberOfWorkersElement === null || currentNumberOfWorkersElement === undefined || currentNumberOfWorkersElement.length === 0) === false) {
              var isNil = false;
              var nilAttribute = js2xml.getAttribute(responseDoc, currentNumberOfWorkersElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute) {
                isNil = nilAttribute === 'true';
              }
              if (isNil === false) {
                var currentNumberOfWorkersInstance = parseInt(currentNumberOfWorkersElement, 10);
                webSpaceInstance.currentNumberOfWorkers = currentNumberOfWorkersInstance;
              }
            }
            
            var currentWorkerSizeElement = js2xml.getElement(responseDoc, webSpacesElement, 'CURRENTWORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (currentWorkerSizeElement && (currentWorkerSizeElement === null || currentWorkerSizeElement === undefined || currentWorkerSizeElement.length === 0) === false) {
              var isNil2 = false;
              var nilAttribute2 = js2xml.getAttribute(responseDoc, currentWorkerSizeElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute2) {
                isNil2 = nilAttribute2 === 'true';
              }
              if (isNil2 === false) {
                var currentWorkerSizeInstance = currentWorkerSizeElement;
                webSpaceInstance.currentWorkerSize = currentWorkerSizeInstance;
              }
            }
            
            var geoLocationElement = js2xml.getElement(responseDoc, webSpacesElement, 'GEOLOCATION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (geoLocationElement) {
              var geoLocationInstance = geoLocationElement;
              webSpaceInstance.geoLocation = geoLocationInstance;
            }
            
            var geoRegionElement = js2xml.getElement(responseDoc, webSpacesElement, 'GEOREGION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (geoRegionElement) {
              var geoRegionInstance = geoRegionElement;
              webSpaceInstance.geoRegion = geoRegionInstance;
            }
            
            var nameElement = js2xml.getElement(responseDoc, webSpacesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              webSpaceInstance.name = nameInstance;
            }
            
            var planElement = js2xml.getElement(responseDoc, webSpacesElement, 'PLAN', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (planElement) {
              var planInstance = planElement;
              webSpaceInstance.plan = planInstance;
            }
            
            var statusElement = js2xml.getElement(responseDoc, webSpacesElement, 'STATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (statusElement) {
              var statusInstance = statusElement;
              webSpaceInstance.status = statusInstance;
            }
            
            var subscriptionElement = js2xml.getElement(responseDoc, webSpacesElement, 'SUBSCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (subscriptionElement) {
              var subscriptionInstance = subscriptionElement;
              webSpaceInstance.subscription = subscriptionInstance;
            }
            
            var workerSizeElement = js2xml.getElement(responseDoc, webSpacesElement, 'WORKERSIZE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (workerSizeElement && (workerSizeElement === null || workerSizeElement === undefined || workerSizeElement.length === 0) === false) {
              var workerSizeInstance = workerSizeElement;
              webSpaceInstance.workerSize = workerSizeInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Get the available geo regions for this web space.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.listGeoRegions = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces?properties=georegions';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { geoRegions: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var geoRegionsSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'GEOREGIONS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (geoRegionsSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, geoRegionsSequenceElement, 'GEOREGION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var geoRegionsElement = js2xml.getElements(responseDoc, geoRegionsSequenceElement, 'GEOREGION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var geoRegionInstance = {};
            result.geoRegions.push(geoRegionInstance);
            
            var descriptionElement = js2xml.getElement(responseDoc, geoRegionsElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (descriptionElement) {
              var descriptionInstance = descriptionElement;
              geoRegionInstance.description = descriptionInstance;
            }
            
            var nameElement = js2xml.getElement(responseDoc, geoRegionsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              geoRegionInstance.name = nameInstance;
            }
            
            var sortOrderElement = js2xml.getElement(responseDoc, geoRegionsElement, 'SORTORDER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sortOrderElement && (sortOrderElement === null || sortOrderElement === undefined || sortOrderElement.length === 0) === false) {
              var isNil = false;
              var nilAttribute = js2xml.getAttribute(responseDoc, sortOrderElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute) {
                isNil = nilAttribute === 'true';
              }
              if (isNil === false) {
                var sortOrderInstance = parseInt(sortOrderElement, 10);
                geoRegionInstance.sortOrder = sortOrderInstance;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Get the source control users allowed to publish to this web space.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.listPublishingUsers = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces?properties=publishingUsers';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { users: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var arrayOfstringSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'ARRAYOFSTRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS');
        if (arrayOfstringSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, arrayOfstringSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex1 = loweredIndex1 + 1) {
            var arrayOfstringElement = js2xml.getElements(responseDoc, arrayOfstringSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex1];
            var stringInstance = {};
            result.users.push(stringInstance);
            
            var stringInstance2 = arrayOfstringElement;
            stringInstance.name = stringInstance2;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * You can retrieve a list of all web sites in a web space by issuing an HTTP
   * GET request.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/dn236429.aspx for
   * more information)
   * 
   * @param {string} webSpaceName The name of the web space.
   * 
   * @param {WebSiteListParameters} [parametersOrCallback] Additional
   * parameters.
   * 
   * @param {array} [parametersOrCallback.propertiesToInclude] Specifies a list
   * of the names of any additional parameters to return.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  WebSpaceOperations.prototype.listWebSites = function(webSpaceName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (webSpaceName === null || webSpaceName === undefined) {
      return callback(new Error('webSpaceName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + (this.client.credentials.subscriptionId ? this.client.credentials.subscriptionId.trim() : '') + '/services/WebSpaces/' + webSpaceName.trim() + '/sites?';
    if (parameters && parameters.propertiesToInclude && parameters.propertiesToInclude.length > 0) {
      url2 = url2 + '&propertiesToInclude=' + encodeURIComponent(parameters.propertiesToInclude.join(','));
    }
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { webSites: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var sitesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SITES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (sitesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, sitesSequenceElement, 'SITE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var sitesElement = js2xml.getElements(responseDoc, sitesSequenceElement, 'SITE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var siteInstance = { enabledHostNames: [], hostNames: [], hostNameSslStates: [], sslCertificates: [] };
            result.webSites.push(siteInstance);
            
            var adminEnabledElement = js2xml.getElement(responseDoc, sitesElement, 'ADMINENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (adminEnabledElement) {
              var adminEnabledInstance = adminEnabledElement === 'true';
              siteInstance.adminEnabled = adminEnabledInstance;
            }
            
            var availabilityStateElement = js2xml.getElement(responseDoc, sitesElement, 'AVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (availabilityStateElement) {
              var availabilityStateInstance = availabilityStateElement;
              siteInstance.availabilityState = availabilityStateInstance;
            }
            
            var computeModeElement = js2xml.getElement(responseDoc, sitesElement, 'COMPUTEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (computeModeElement) {
              var computeModeInstance = computeModeElement;
              siteInstance.computeMode = computeModeInstance;
            }
            
            var enabledElement = js2xml.getElement(responseDoc, sitesElement, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (enabledElement) {
              var enabledInstance = enabledElement === 'true';
              siteInstance.enabled = enabledInstance;
            }
            
            var enabledHostNamesSequenceElement = js2xml.getElement(responseDoc, sitesElement, 'ENABLEDHOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (enabledHostNamesSequenceElement) {
              for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex2 = loweredIndex2 + 1) {
                var enabledHostNamesElement = js2xml.getElements(responseDoc, enabledHostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex2];
                siteInstance.enabledHostNames.push(enabledHostNamesElement);
              }
            }
            
            var hostNameSslStatesSequenceElement = js2xml.getElement(responseDoc, sitesElement, 'HOSTNAMESSLSTATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (hostNameSslStatesSequenceElement) {
              for (var loweredIndex3 = 0; loweredIndex3 < js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex3 = loweredIndex3 + 1) {
                var hostNameSslStatesElement = js2xml.getElements(responseDoc, hostNameSslStatesSequenceElement, 'WEBSITEHOSTNAMESSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex3];
                var webSiteHostNameSslStateInstance = {};
                siteInstance.hostNameSslStates.push(webSiteHostNameSslStateInstance);
                
                var nameElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (nameElement) {
                  var nameInstance = nameElement;
                  webSiteHostNameSslStateInstance.name = nameInstance;
                }
                
                var sslStateElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'SSLSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (sslStateElement) {
                  var sslStateInstance = sslStateElement;
                  webSiteHostNameSslStateInstance.sslState = sslStateInstance;
                }
                
                var thumbprintElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (thumbprintElement) {
                  var isNil = false;
                  var nilAttribute = js2xml.getAttribute(responseDoc, thumbprintElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute) {
                    isNil = nilAttribute === 'true';
                  }
                  if (isNil === false) {
                    var thumbprintInstance = thumbprintElement;
                    webSiteHostNameSslStateInstance.thumbprint = thumbprintInstance;
                  }
                }
                
                var virtualIPElement = js2xml.getElement(responseDoc, hostNameSslStatesElement, 'VIRTUALIP', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (virtualIPElement) {
                  var isNil2 = false;
                  var nilAttribute2 = js2xml.getAttribute(responseDoc, virtualIPElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                  if (nilAttribute2) {
                    isNil2 = nilAttribute2 === 'true';
                  }
                  if (isNil2 === false) {
                    var virtualIPInstance = virtualIPElement;
                    webSiteHostNameSslStateInstance.virtualIP = virtualIPInstance;
                  }
                }
              }
            }
            
            var hostNamesSequenceElement = js2xml.getElement(responseDoc, sitesElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (hostNamesSequenceElement) {
              for (var loweredIndex4 = 0; loweredIndex4 < js2xml.getElements(responseDoc, hostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex4 = loweredIndex4 + 1) {
                var hostNamesElement = js2xml.getElements(responseDoc, hostNamesSequenceElement, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex4];
                siteInstance.hostNames.push(hostNamesElement);
              }
            }
            
            var lastModifiedTimeUtcElement = js2xml.getElement(responseDoc, sitesElement, 'LASTMODIFIEDTIMEUTC', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (lastModifiedTimeUtcElement) {
              var lastModifiedTimeUtcInstance = new Date(lastModifiedTimeUtcElement);
              siteInstance.lastModifiedTimeUtc = lastModifiedTimeUtcInstance;
            }
            
            var nameElement2 = js2xml.getElement(responseDoc, sitesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement2) {
              var nameInstance2 = nameElement2;
              siteInstance.name = nameInstance2;
            }
            
            var ownerElement = js2xml.getElement(responseDoc, sitesElement, 'OWNER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (ownerElement) {
              var isNil3 = false;
              var nilAttribute3 = js2xml.getAttribute(responseDoc, ownerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute3) {
                isNil3 = nilAttribute3 === 'true';
              }
              if (isNil3 === false) {
                var ownerInstance = ownerElement;
                siteInstance.owner = ownerInstance;
              }
            }
            
            var repositorySiteNameElement = js2xml.getElement(responseDoc, sitesElement, 'REPOSITORYSITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (repositorySiteNameElement) {
              var repositorySiteNameInstance = repositorySiteNameElement;
              siteInstance.repositorySiteName = repositorySiteNameInstance;
            }
            
            var runtimeAvailabilityStateElement = js2xml.getElement(responseDoc, sitesElement, 'RUNTIMEAVAILABILITYSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (runtimeAvailabilityStateElement) {
              var runtimeAvailabilityStateInstance = runtimeAvailabilityStateElement;
              siteInstance.runtimeAvailabilityState = runtimeAvailabilityStateInstance;
            }
            
            var sSLCertificatesSequenceElement = js2xml.getElement(responseDoc, sitesElement, 'SSLCERTIFICATES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sSLCertificatesSequenceElement) {
              var isNil4 = false;
              var nilAttribute4 = js2xml.getAttribute(responseDoc, sSLCertificatesSequenceElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute4) {
                isNil4 = nilAttribute4 === 'true';
              }
              if (isNil4 === false) {
                for (var loweredIndex5 = 0; loweredIndex5 < js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex5 = loweredIndex5 + 1) {
                  var sSLCertificatesElement = js2xml.getElements(responseDoc, sSLCertificatesSequenceElement, 'CERTIFICATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex5];
                  var certificateInstance = { hostNames: [] };
                  siteInstance.sslCertificates.push(certificateInstance);
                  
                  var expirationDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'EXPIRATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (expirationDateElement && (expirationDateElement === null || expirationDateElement === undefined || expirationDateElement.length === 0) === false) {
                    var isNil5 = false;
                    var nilAttribute5 = js2xml.getAttribute(responseDoc, expirationDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute5) {
                      isNil5 = nilAttribute5 === 'true';
                    }
                    if (isNil5 === false) {
                      var expirationDateInstance = new Date(expirationDateElement);
                      certificateInstance.expirationDate = expirationDateInstance;
                    }
                  }
                  
                  var friendlyNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'FRIENDLYNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (friendlyNameElement) {
                    var isNil6 = false;
                    var nilAttribute6 = js2xml.getAttribute(responseDoc, friendlyNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute6) {
                      isNil6 = nilAttribute6 === 'true';
                    }
                    if (isNil6 === false) {
                      var friendlyNameInstance = friendlyNameElement;
                      certificateInstance.friendlyName = friendlyNameInstance;
                    }
                  }
                  
                  var hostNamesSequenceElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'HOSTNAMES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (hostNamesSequenceElement2) {
                    var isNil7 = false;
                    var nilAttribute7 = js2xml.getAttribute(responseDoc, hostNamesSequenceElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute7) {
                      isNil7 = nilAttribute7 === 'true';
                    }
                    if (isNil7 === false) {
                      for (var loweredIndex6 = 0; loweredIndex6 < js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS').length; loweredIndex6 = loweredIndex6 + 1) {
                        var hostNamesElement2 = js2xml.getElements(responseDoc, hostNamesSequenceElement2, 'STRING', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/ARRAYS')[loweredIndex6];
                        certificateInstance.hostNames.push(hostNamesElement2);
                      }
                    }
                  }
                  
                  var issueDateElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUEDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (issueDateElement && (issueDateElement === null || issueDateElement === undefined || issueDateElement.length === 0) === false) {
                    var isNil8 = false;
                    var nilAttribute8 = js2xml.getAttribute(responseDoc, issueDateElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute8) {
                      isNil8 = nilAttribute8 === 'true';
                    }
                    if (isNil8 === false) {
                      var issueDateInstance = new Date(issueDateElement);
                      certificateInstance.issueDate = issueDateInstance;
                    }
                  }
                  
                  var issuerElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'ISSUER', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (issuerElement) {
                    var isNil9 = false;
                    var nilAttribute9 = js2xml.getAttribute(responseDoc, issuerElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute9) {
                      isNil9 = nilAttribute9 === 'true';
                    }
                    if (isNil9 === false) {
                      var issuerInstance = issuerElement;
                      certificateInstance.issuer = issuerInstance;
                    }
                  }
                  
                  var passwordElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PASSWORD', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (passwordElement) {
                    var isNil10 = false;
                    var nilAttribute10 = js2xml.getAttribute(responseDoc, passwordElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute10) {
                      isNil10 = nilAttribute10 === 'true';
                    }
                    if (isNil10 === false) {
                      var passwordInstance = passwordElement;
                      certificateInstance.password = passwordInstance;
                    }
                  }
                  
                  var pfxBlobElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'PFXBLOB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (pfxBlobElement) {
                    var isNil11 = false;
                    var nilAttribute11 = js2xml.getAttribute(responseDoc, pfxBlobElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute11) {
                      isNil11 = nilAttribute11 === 'true';
                    }
                    if (isNil11 === false) {
                      var pfxBlobInstance = new Buffer(pfxBlobElement, 'base64').toString('utf8');
                      certificateInstance.pfxBlob = pfxBlobInstance;
                    }
                  }
                  
                  var selfLinkElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (selfLinkElement) {
                    var isNil12 = false;
                    var nilAttribute12 = js2xml.getAttribute(responseDoc, selfLinkElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute12) {
                      isNil12 = nilAttribute12 === 'true';
                    }
                    if (isNil12 === false) {
                      var selfLinkInstance = selfLinkElement;
                      certificateInstance.selfLinkUri = selfLinkInstance;
                    }
                  }
                  
                  var siteNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SITENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (siteNameElement) {
                    var isNil13 = false;
                    var nilAttribute13 = js2xml.getAttribute(responseDoc, siteNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute13) {
                      isNil13 = nilAttribute13 === 'true';
                    }
                    if (isNil13 === false) {
                      var siteNameInstance = siteNameElement;
                      certificateInstance.siteName = siteNameInstance;
                    }
                  }
                  
                  var subjectNameElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'SUBJECTNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (subjectNameElement) {
                    var isNil14 = false;
                    var nilAttribute14 = js2xml.getAttribute(responseDoc, subjectNameElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute14) {
                      isNil14 = nilAttribute14 === 'true';
                    }
                    if (isNil14 === false) {
                      var subjectNameInstance = subjectNameElement;
                      certificateInstance.subjectName = subjectNameInstance;
                    }
                  }
                  
                  var thumbprintElement2 = js2xml.getElement(responseDoc, sSLCertificatesElement, 'THUMBPRINT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (thumbprintElement2) {
                    var isNil15 = false;
                    var nilAttribute15 = js2xml.getAttribute(responseDoc, thumbprintElement2, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute15) {
                      isNil15 = nilAttribute15 === 'true';
                    }
                    if (isNil15 === false) {
                      var thumbprintInstance2 = thumbprintElement2;
                      certificateInstance.thumbprint = thumbprintInstance2;
                    }
                  }
                  
                  var toDeleteElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'TODELETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (toDeleteElement && (toDeleteElement === null || toDeleteElement === undefined || toDeleteElement.length === 0) === false) {
                    var isNil16 = false;
                    var nilAttribute16 = js2xml.getAttribute(responseDoc, toDeleteElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute16) {
                      isNil16 = nilAttribute16 === 'true';
                    }
                    if (isNil16 === false) {
                      var toDeleteInstance = toDeleteElement === 'true';
                      certificateInstance.isToBeDeleted = toDeleteInstance;
                    }
                  }
                  
                  var validElement = js2xml.getElement(responseDoc, sSLCertificatesElement, 'VALID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  if (validElement && (validElement === null || validElement === undefined || validElement.length === 0) === false) {
                    var isNil17 = false;
                    var nilAttribute17 = js2xml.getAttribute(responseDoc, validElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                    if (nilAttribute17) {
                      isNil17 = nilAttribute17 === 'true';
                    }
                    if (isNil17 === false) {
                      var validInstance = validElement === 'true';
                      certificateInstance.isValid = validInstance;
                    }
                  }
                }
              }
            }
            
            var selfLinkElement2 = js2xml.getElement(responseDoc, sitesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (selfLinkElement2) {
              var selfLinkInstance2 = selfLinkElement2;
              siteInstance.uri = selfLinkInstance2;
            }
            
            var serverFarmElement = js2xml.getElement(responseDoc, sitesElement, 'SERVERFARM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serverFarmElement) {
              var serverFarmInstance = serverFarmElement;
              siteInstance.serverFarm = serverFarmInstance;
            }
            
            var siteModeElement = js2xml.getElement(responseDoc, sitesElement, 'SITEMODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (siteModeElement) {
              var siteModeInstance = siteModeElement;
              siteInstance.siteMode = siteModeInstance;
            }
            
            var sitePropertiesElement = js2xml.getElement(responseDoc, sitesElement, 'SITEPROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sitePropertiesElement) {
              var sitePropertiesInstance = { appSettings: {}, metadata: {}, properties: {} };
              siteInstance.siteProperties = sitePropertiesInstance;
              
              var appSettingsSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'APPSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (appSettingsSequenceElement) {
                for (var loweredIndex7 = 0; loweredIndex7 < js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex7 = loweredIndex7 + 1) {
                  var appSettingsElement = js2xml.getElements(responseDoc, appSettingsSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex7];
                  var appSettingsKey = js2xml.getElement(responseDoc, appSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  var appSettingsValue = js2xml.getElement(responseDoc, appSettingsElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  sitePropertiesInstance.appSettings[appSettingsKey] = appSettingsValue;
                }
              }
              
              var metadataSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'METADATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (metadataSequenceElement) {
                for (var loweredIndex8 = 0; loweredIndex8 < js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex8 = loweredIndex8 + 1) {
                  var metadataElement = js2xml.getElements(responseDoc, metadataSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex8];
                  var metadataKey = js2xml.getElement(responseDoc, metadataElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  var metadataValue = js2xml.getElement(responseDoc, metadataElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  sitePropertiesInstance.metadata[metadataKey] = metadataValue;
                }
              }
              
              var propertiesSequenceElement = js2xml.getElement(responseDoc, sitePropertiesElement, 'PROPERTIES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (propertiesSequenceElement) {
                for (var loweredIndex9 = 0; loweredIndex9 < js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex9 = loweredIndex9 + 1) {
                  var propertiesElement = js2xml.getElements(responseDoc, propertiesSequenceElement, 'NAMEVALUEPAIR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex9];
                  var propertiesKey = js2xml.getElement(responseDoc, propertiesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  var propertiesValue = js2xml.getElement(responseDoc, propertiesElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                  sitePropertiesInstance.properties[propertiesKey] = propertiesValue;
                }
              }
            }
            
            var stateElement = js2xml.getElement(responseDoc, sitesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              siteInstance.state = stateInstance;
            }
            
            var usageStateElement = js2xml.getElement(responseDoc, sitesElement, 'USAGESTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (usageStateElement) {
              var usageStateInstance = usageStateElement;
              siteInstance.usageState = usageStateInstance;
            }
            
            var webSpaceElement = js2xml.getElement(responseDoc, sitesElement, 'WEBSPACE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (webSpaceElement) {
              var webSpaceInstance = webSpaceElement;
              siteInstance.webSpace = webSpaceInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return WebSpaceOperations;
})();
